--------------------------------------------------------------------------
-- Play this script in SCHEMA_315@SCHEMA to make it look like SCHEMA_320@SCHEMA
--                                                                      --
-- Please review the script before using it to make sure it won't       --
-- cause any unacceptable data loss.                                    --
--                                                                      --
-- SCHEMA_315@SCHEMA Schema Extracted by User SCHEMA_315
-- SCHEMA_320@SCHEMA Schema Extracted by User SCHEMA_320
--------------------------------------------------------------------------

create or replace
package body rhn_entitlements
is
	function entitlement_grants_service (
	    entitlement_in in varchar2,
	    service_level_in in varchar2
	) return number	is
	begin
	    if service_level_in = 'provisioning' then

		if entitlement_in = 'provisioning_entitled' then
		    return 1;
		else
		    return 0;
		end if;
	    elsif service_level_in = 'management' then

		if entitlement_in = 'provisioning_entitled' then
		    return 1;
		elsif entitlement_in = 'enterprise_entitled' then
		    return 1;
		elsif entitlement_in = 'nonlinux_entitled' then
		    return 1;
		else
		    return 0;
		end if;

	    elsif service_level_in = 'updates' then
		return 1;

	    else
		return 0;
	    end if;

	end entitlement_grants_service;


	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		cursor server_groups is
			select	sg.id				server_group_id
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = type_label_in
				and sgt.id = sg.group_type
				and sg.org_id = org_id_in;
	begin
		for sg in server_groups loop
			return sg.server_group_id;
		end loop;
		return rhn_entitlements.create_entitlement_group(
				org_id_in,
				type_label_in
			);
	end lookup_entitlement_group;

	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		sg_id_val number;
	begin
		select	rhn_server_group_id_seq.nextval
		into	sg_id_val
		from	dual;

		insert into rhnServerGroup (
				id, name, description, max_members, current_members,
				group_type, org_id
			) (
				select	sg_id_val, sgt.label, sgt.label,
						0, 0, sgt.id, org_id_in
				from	rhnServerGroupType sgt
				where	sgt.label = type_label_in
			);

		return sg_id_val;
	end create_entitlement_group;

	function get_server_entitlement (
		server_id_in in number
	) return varchar2 is
		cursor server_groups is
			select	sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_id = server_id_in
				and sg.id = sgm.server_group_id
				and sgt.id = sg.group_type
				and sgt.label in ('sw_mgr_entitled','enterprise_entitled',
					'provisioning_entitled', 'nonlinux_entitled');
	begin
		for sg in server_groups loop
			return sg.label;
		end loop;
		return 'none';
	end get_server_entitlement;

	procedure unentitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
		group_id number;
	begin
		select	sg.id
		into	group_id
		from	rhnServerGroupType sgt,
				rhnServerGroup sg,
				rhnServerGroupMembers sgm,
				rhnServer s
		where	s.id = server_id_in
			and s.id = sgm.server_id
			and sgm.server_group_id = sg.id
			and sg.org_id = s.org_id
			and sgt.label = type_label_in
			and sgt.id = sg.group_type;

		rhn_server.delete_from_servergroup(server_id_in, group_id);
		exception
		when no_data_found then
		    null;
	end unentitle_server;

	-- right now, this is racy.  Don't do it twice at once
	-- for the same org.  You've been warned...
	procedure upgrade_server (
		server_id_in in number
	) is
		old_sgid number;
		new_sgid number;
		new_sg_max number;
		new_sg_used number;
	begin
		begin
			select	sg0.id, sg1.id,
					sg1.max_members, sg1.current_members
			into	old_sgid, new_sgid,
					new_sg_max, new_sg_used
			from	rhnServerGroup sg1,
					rhnServerGroupType sgt1,
					rhnServerGroupMembers sgm,
					rhnServerGroup sg0,
					rhnServerGroupType sgt0,
					rhnServer s
			where	s.id = server_id_in
				and s.org_id = sg0.org_id
				and sgt0.label = 'enterprise_entitled'
				and sgt0.id = sg0.group_type
				and s.id = sgm.server_id
				and sgm.server_group_id = sg0.id
				and s.org_id = sg1.org_id
				and sgt1.label = 'provisioning_entitled'
				and sgt1.id = sg1.group_type;
		exception
			when no_data_found then
				rhn_exception.raise_exception('invalid_upgrade');
		end;
		-- it could still happen that you run out after we check this.
		-- that's what the comment above is all about...
		if new_sg_used >= new_sg_max then
			rhn_exception.raise_exception('servergroup_max_members');
		end if;
		rhn_server.delete_from_servergroup(server_id_in, old_sgid);
		rhn_server.insert_into_servergroup(server_id_in, new_sgid);
	end upgrade_server;

	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
		cursor servergroups is
			select	sg.id id,
					s.server_arch_id server_arch_id,
					sgt.id server_group_type
			from	rhnServerGroupType sgt,
					rhnServerGroup sg,
					rhnServer s
			where	s.id = server_id_in
				and s.org_id = sg.org_id
				and sgt.label = type_label_in
				and sg.group_type = sgt.id;
		cursor sg_arches(sgt_id_in in number, sa_id_in in number) is
			select	1
			from	rhnServerServerGroupArchCompat
			where	server_group_type = sgt_id_in
				and server_arch_id = sa_id_in;
		previous_ent rhnServerGroupType.label%TYPE;
	begin
		previous_ent := rhn_entitlements.get_server_entitlement(server_id_in);
		if previous_ent != 'none' then
			rhn_exception.raise_exception(
								'no_server_multiple_swmgr_entitlements');
		end if;

		for sg in servergroups loop
			for a in sg_arches(sg.server_group_type, sg.server_arch_id) loop
				-- if the max_members is too low, this'll raise
				-- servergroup_max_members
				rhn_server.insert_into_servergroup(server_id_in, sg.id);
				return;
			end loop;
			-- we only get here if we can't find any compatible
			-- server_arch+server_group_type combination
			rhn_exception.raise_exception('mismatching_entitlement');
		end loop;
		-- we only get here if there's _no_ server group for that type;
		-- that's the same as max_members=0
		rhn_exception.raise_exception('servergroup_max_members');
	end;

	procedure set_customer_enterprise (
		customer_id in number
	) is
		entitlement_type_id number;
		status number;
	begin
		select	roet.id, roe.org_id
		into	entitlement_type_id, status
		from	rhnOrgEntitlementType	roet,
				rhnOrgEntitlements		roe
		where	1=1
			and roet.label = 'sw_mgr_enterprise'
			and roet.id = roe.entitlement_id(+)
			and roe.org_id(+) = customer_id;

		if status is null then
			insert into rhnOrgEntitlements (
					org_id, entitlement_id
				) values (
					customer_id, entitlement_type_id
			);
		end if;

		-- XXX is org_applicant even right to create anymore?
		update		rhnUserGroup
			set		max_members = null
			where	1=1
				and org_id = customer_id
				and group_type in (
					select	id
					from	rhnUserGroupType
					where	label in ('org_admin', 'org_applicant')
				);
	end set_customer_enterprise;

	procedure create_config_admin (
		customer_id_in in number
	) is
		cursor config_admin_groups is
			select	1
			from	rhnUserGroup ug,
				rhnUserGroupType ugt
			where	ugt.label = 'config_admin'
				and ug.org_id = customer_id_in
				and ugt.id = ug.group_type;
	begin
		for config_admin_group in config_admin_groups loop
			return;
		end loop;

		insert into rhnUserGroup (
			id, name,
			description,
			max_members, group_type, org_id
		) (
			select	rhn_user_group_id_seq.nextval,
					'Configuration Administrator',
					'Configuration Administrator for Org ' || ' (1)',
					NULL, id, 1
			from	rhnUserGroupType
			where	label = 'config_admin'
		);
	end create_config_admin;

	procedure set_customer_provisioning (
		customer_id_in in number
	) is
		cursor entitlements is
			select	roet.id entitlement_type_id,
					roe.org_id status
			from	rhnOrgEntitlements		roe,
					rhnOrgEntitlementType	roet
			where	1=1
				and roet.label in ('rhn_provisioning','sw_mgr_enterprise')
				and roet.id = roe.entitlement_id(+)
				and roe.org_id(+) = customer_id_in;
	begin
		for e in entitlements loop
			if e.status is null then
				insert into rhnOrgEntitlements (
						org_id, entitlement_id
					) values (
						customer_id_in, e.entitlement_type_id
					);
			end if;
		end loop;
		if is_satellite() = 1 then
			rhn_entitlements.create_config_admin(customer_id_in);
		end if;
	end set_customer_provisioning;

	procedure set_customer_nonlinux (
		customer_id_in in number
	) is
		cursor entitlements is
			select	roet.id entitlement_type_id,
					roe.org_id status
			from	rhnOrgEntitlements		roe,
					rhnOrgEntitlementType	roet
			where	1=1
				and roet.label in ('rhn_nonlinux')
				and roet.id = roe.entitlement_id(+)
				and roe.org_id(+) = customer_id_in;
	begin
		for e in entitlements loop
			if e.status is null then
				insert into rhnOrgEntitlements (
						org_id, entitlement_id
					) values (
						customer_id_in, e.entitlement_type_id
					);
			end if;
		end loop;
		if is_satellite() = 1 then
			rhn_entitlements.create_config_admin(customer_id_in);
		end if;
	end set_customer_nonlinux;

	procedure unset_customer_enterprise (
		customer_id in number
	) is
	begin
		delete	from	rhnOrgEntitlements
				where	1=1
					and org_id = customer_id
					and entitlement_id in (
						select	id
						from	rhnOrgEntitlementType
						where	label = 'sw_mgr_enterprise'
					);

		-- XXX there's no way this is right.  It probably should be pruning
		-- these groups to 0 and *removing* them (and anybody in
		-- org_applicant?).  But then, maybe not.  ISTR something about
		-- never becomming un-enterprise
		update		rhnUserGroup
			set		max_members = null
			where	1=1
				and org_id = customer_id
				and group_type in (
					select	id
					from	rhnUserGroupType
					where	label in ('org_admin', 'org_applicant')
				);
	end unset_customer_enterprise;

	procedure unset_customer_provisioning (
		customer_id_in in number
	) is
		usergroup number;
	begin
		delete from rhnOrgEntitlements
			where	1=1
				and org_id = customer_id_in
				and entitlement_id in (
					select	id
					from	rhnOrgEntitlementType
					where	label = 'rhn_provisioning'
				);
	end;

	procedure unset_customer_nonlinux (
		customer_id_in in number
	) is
		usergroup number;
	begin
		delete from rhnOrgEntitlements
			where	1=1
				and org_id = customer_id_in
				and entitlement_id in (
					select	id
					from	rhnOrgEntitlementType
					where	label = 'rhn_nonlinux'
				);
	end;

	procedure prune_group (
		group_id_in in number,
		type_in in char,
		quantity_in in number
	) is
		cursor usergroups is
			select	user_id, user_group_id, ugt.label
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	1=1
				and ugm.user_group_id = group_id_in
				and ugm.user_id in (
					select	user_id
					from	(
						select	rownum row_number,
								user_id,
								time
						from	(
							select	user_id,
									modified time
							from	rhnUserGroupMembers
							where	user_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id;
		cursor servergroups is
			select	server_id, server_group_id, sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_group_id = group_id_in
				and sgm.server_id in (
					select	server_id
					from	(
						select	rownum row_number,
								server_id,
								time
						from	(
							select	server_id,
									modified time
							from	rhnServerGroupMembers
							where	server_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and sgm.server_group_id = sg.id
				and sg.group_type = sgt.id;
	begin
		if type_in = 'U' then
			update		rhnUserGroup
				set		max_members = quantity_in
				where	id = group_id_in;

			for ug in usergroups loop
				-- rhnUserGroup.max_members gets fixed in a trigger, no
				-- need to do that here
				delete
					from	rhnUserGroupMembers
					where	user_group_id = ug.user_group_id
						and user_id = ug.user_id;
			end loop;
		elsif type_in = 'S' then
			update		rhnServerGroup
				set		max_members = quantity_in
				where	id = group_id_in;

			for sg in servergroups loop
				unentitle_server(sg.server_id, sg.label);
				rhn_channel.clear_subscriptions(sg.server_id);
			end loop;
		end if;
	end prune_group;

	procedure set_group_count (
		customer_id_in in number,
		type_in in char,
		group_type_in in number,
		quantity_in in number
	) is
		group_id number;
		quantity number;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;

		if type_in = 'U' then
			select	rug.id
			into	group_id
			from	rhnUserGroup rug
			where	1=1
				and rug.org_id = customer_id_in
				and rug.group_type = group_type_in;
		elsif type_in = 'S' then
			select	rsg.id
			into	group_id
			from	rhnServerGroup rsg
			where	1=1
				and rsg.org_id = customer_id_in
				and rsg.group_type = group_type_in;
		end if;

		rhn_entitlements.prune_group(
			group_id,
			type_in,
			quantity
		);
	exception
		when no_data_found then
			if type_in = 'U' then
				insert into rhnUserGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_user_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnUserGroupType
						where	id = group_type_in
				);
			elsif type_in = 'S' then
				insert into rhnServerGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_server_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnServerGroupType
						where	id = group_type_in
				);
			end if;
	end set_group_count;

	procedure prune_family (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor serverchannels is
			select	sc.server_id,
					sc.channel_id
			from	rhnServerChannel sc,
					rhnChannelFamilyMembers cfm
			where	1=1
				and cfm.channel_family_id = channel_family_id_in
				and cfm.channel_id = sc.channel_id
				and server_id in (
					select	server_id
					from	(
						select	server_id,
								time,
								rownum row_number
						from	(
							select	rs.id					server_id,
									rcfm.modified			time
							from	rhnServer				rs,
									rhnServerChannel		rsc,
									rhnChannelFamilyMembers	rcfm
							where	1=1
								and rs.org_id = customer_id_in
								and rs.id = rsc.server_id
								and rsc.channel_id = rcfm.channel_id
								and rcfm.channel_family_id =
									channel_family_id_in
							order by time asc
						)
					)
					where row_number > quantity_in
				);
	begin
		-- if we get a null customer_id, this is completely bogus.
		if customer_id_in is null then
			return;
		end if;

		update		rhnChannelFamilyPermissions
			set		max_members = quantity_in
			where	1=1
				and org_id = customer_id_in
				and channel_family_id = channel_family_id_in;

		for sc in serverchannels loop
			rhn_channel.unsubscribe_server(sc.server_id, sc.channel_id, 1, 1);
		end loop;
	end prune_family;

	procedure set_family_count (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		perm_exists number;
		quantity number;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;

		-- if the perm _doesn't_ exist, we get the exception
		select	1
		into	perm_exists
		from	rhnChannelFamilyPermissions
		where	1=1
			and channel_family_id = channel_family_id_in
			and (org_id = customer_id_in or
				(org_id is null and customer_id_in is null));

		rhn_entitlements.prune_family(
				customer_id_in,
				channel_family_id_in,
				quantity
			);
	exception
		when no_data_found then
			insert into rhnChannelFamilyPermissions (
					channel_family_id, org_id, max_members, current_members
				) values (
					channel_family_id_in, customer_id_in, quantity, 0
				);
	end set_family_count;

	-- this expects quantity_in to be the number of available slots, not the
	-- max_members of the server group.  If you give it too many, it'll fail
	-- and raise servergroup_max_members.
	-- We should NEVER run this unless we're SURE that we won't
	-- be violating the max.
	procedure entitle_last_modified_servers (
		customer_id_in in number,
		type_label_in in varchar2,
		quantity_in in number
	) is
		-- find the servers that aren't currently in slots
		cursor servers(cid_in in number, quant_in in number) is
			select	server_id
			from	(
						select	rownum row_number,
								server_id
						from	(
									select	rs.id server_iD
									from	rhnServer rs
									where	1=1
										and rs.org_id = cid_in
										and not exists (
											select	1
											from	rhnServerGroup sg,
													rhnServerGroupMembers rsgm
											where	rsgm.server_id = rs.id
												and rsgm.server_group_id = sg.id
												and sg.group_type is not null
										)
									order by modified desc
								)
					)
			where	row_number <= quant_in;
	begin
		for server in servers(customer_id_in, quantity_in) loop
			rhn_entitlements.entitle_server(server.server_id, type_label_in);
		end loop;
	end entitle_last_modified_servers;

	procedure get_available_slots (
		customer_id_in in number,
		type_label_in in varchar2,
		server_group_id_out out number,
		available_out out number,
		upgrade_out out number
	) is
		mgmt_slots_unused number;
		mgmt_slots_used number;
		prov_slots_unused number;
		prov_slots_used number;
		swmgr_slots_unused number;
		sgid number;
	begin
		-- if it's the simple case, do it and get out
		if type_label_in in ('sw_mgr_entitled','nonlinux_entitled') then
			begin
				select	max_members - current_members, sg.id
				into	swmgr_slots_unused, sgid
				from	rhnServerGroup		sg,
						rhnServerGroupType	sgt
				where	sgt.label = type_label_in
					and sg.org_id = customer_id_in
					and sg.group_type = sgt.id;

				available_out := swmgr_slots_unused;
				upgrade_out := 0;
				server_group_id_out := sgid;
			exception
				when no_data_found then
					available_out := 0;
					if type_label_in = 'sw_mgr_entitled' then
						available_out := 1;
					end if;
					upgrade_out := 0;
					server_group_id_out := null;
			end;
			return;
		end if;

		begin
			select	sg.current_members,
					sg.max_members - sg.current_members,
					sg.id
			into	mgmt_slots_used,
					mgmt_slots_unused,
					sgid
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = 'enterprise_entitled'
				and sg.org_id = customer_id_in
				and sg.group_type = sgt.id;
		exception
			when no_data_found then
				-- if there's no enterprise_entitled group, then
				-- this user doesn't have mgmt or prov at all,
				-- so bail out now.
				available_out := 0;
				upgrade_out := 0;
				server_group_id_out := null;
				return;
		end;

		-- we list all mgmt slots as things you can upgrade to
		-- from basic
		if type_label_in = 'enterprise_entitled' then
			available_out := mgmt_slots_unused;
			upgrade_out := 0;
			server_group_id_out := sgid;
			return;
		end if;

		-- the only thing left is provisioning
		begin
			select	sg.current_members,
					sg.max_members - sg.current_members,
					sg.id
			into	prov_slots_used,
					prov_slots_unused,
					sgid
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = 'provisioning_entitled'
				and sg.org_id = customer_id_in
				and sg.group_type = sgt.id;
		exception
			when no_data_found then
				available_out := 0;
				upgrade_out := 0;
				server_group_id_out := null;
				return;
		end;

		-- An example to illustrate what I'm doing to compute upgradable slots
		-- 1) The user buys 3 provisioning entitlements.  This means:
		--      enterprise_entitled max_members = 3, current_members = 0    (prov_slots_unused = 3)
		--      provisioning_entitled max_members = 3, current_members = 0  (prov_slots_unsued = 3)
		--    0 upgrade, 3 available
		-- 2) The user subscribes a system for management.  This means:
		--      enterprise_entitled max_members = 3, current_members = 1    (mgmt_slots_unused = 2)
		--      provisioning_entitled max_members = 3, current_members = 0  (prov_slots_unused = 3)
		--    1 upgrade, 2 available
		-- 3) The user subscribes a system for provisioning.  This means:
		--      enterprise_entitled max_members = 2, current_members = 1    (mgmt_slots_unused = 1)
		--      provisioning_entitled max_members = 3, current_members = 1  (prov_slots_unused = 2)
		--    1 upgrade, 1 available
		-- 4) The user subscribes another provisioning.  I.e.:
		--      enterprise_entitled max_members = 1, current_members = 1    (mgmt_slots_unused = 0)
		--      provisioning_entitled max_members = 3, current_members = 2  (prov_slots_unused = 1)
		--    1 upgrade, 0 available
		-- 5) the user upgrades 1 mgmt to prov.  Here:
		--    1) unsubscribe from mgmt
		--      enterprise_entitled max_members = 1, current_members = 0    (mgmt_slots_unused = 1)
		--      provisioning_entitled max_members = 3, current_members = 2  (prov_slots_unused = 1)
		--    0 upgrade, 1 available
		--    2) subscribe to prov
		--      enterprise_entitled max_members = 0, current_members = 0    (mgmt_slots_unused = 0)
		--      provisioning_entitled max_members = 3, current_members = 3  (prov_slots_unused = 0)
		--    that'll result in 0 upgrade, 0 available
		-- 6) user buys 1 mgmt:
		--      enterprise_entitled max_members = 1, current_members = 0    (mgmt_slots_unused = 1)
		--      provisioning_entitled max_members = 3, current_members = 3  (prov_slots_unused = 0)
		--    -1 on upgrade, 1 on available, BORK, should be 0, 0 (special cased with p-m<0)
		-- 7) user registers a management box
		--      enterprise_entitled max_members = 1, current_members = 1    (mgmt_slots_unused = 0)
		--      provisioning_entitled max_members = 3, current_members = 3  (prov_slots_unused = 0)
		--    0, 0, which is right.
		-- 8) user deletes that management box
		--      enterprise_entitled max_members = 1, current_members = 0    (mgmt_slots_unused = 1)
		--      provisioning_entitled max_members = 3, current_members = 3  (prov_slots_unused = 0)
		--    -1 on upgrade, 1 on available, BORK, should be 0, 0 (special cased with p-m<0)
		-- 9) user buys a provisioning slot
		--      enterprise_entitled max_members = 2, current_members = 0    (mgmt_slots_unused = 2)
		--      provisioning_entitled max_members = 4, current_members = 3  (prov_slots_unused = 1)
		--    -1 upgrade, 0 available, BORK.  special case makes it 1 available, 0 upgrade
		-- 10) user registers a mgmt box
		--      enterprise_entitled max_members = 2, current_members = 1    (mgmt_slots_unused = 1)
		--      provisioning_entitled max_members = 4, current_members = 3  (prov_slots_unused = 1)
		--    0 upgrade, 1 available
		if prov_slots_unused - mgmt_slots_unused < 0 then
			-- swmgr_slots_unused is tmp here
			available_out := prov_slots_unused;
			upgrade_out := 0;
		elsif prov_slots_unused > 0 then
			-- if there's one of each, we list it as available
			-- if there's only a prov slot, then it's upgradable.
			upgrade_out := prov_slots_unused - mgmt_slots_unused;
			available_out := prov_slots_unused - (prov_slots_unused - mgmt_slots_unused);
		else
			upgrade_out := 0;
			available_out := 0;
		end if;
		server_group_id_out := sgid;
		return;
	end get_available_slots;

	procedure prune_everything (
		customer_id_in in number
	) is
		cursor everything is
			-- all our server groups
			select	sg.id					id,
					'S'						type,
					sg.max_members			quantity
			from	rhnServerGroup			sg
			where	sg.org_id = customer_id_in
			union
			-- all our user groups
			select	ug.id					id,
					'U'						type,
					ug.max_members 			quantity
			from	rhnUserGroup			ug
			where	ug.org_id = customer_id_in
			union (
			-- all the channel families we have perms to
			select	cfp.channel_family_id	id,
					'C'						type,
					cfp.max_members			quantity
			from	rhnChannelFamilyPermissions cfp
			where	cfp.org_id = customer_id_in
				or	cfp.org_id is null
			union
			-- plus all the ones we'rE Using that we have no perms for
			select	cfm.channel_family_id	id,
					'C'						type,
					0						quantity
			from	rhnChannelFamily		cf,
					rhnChannelFamilyMembers	cfm,
					rhnServerChannel		sc,
					rhnServer				s
			where	s.org_id = customer_id_in
				and s.id = sc.server_id
				and sc.channel_id = cfm.channel_id
				and not exists (
					select	1
					from	rhnChannelFamilyPermissions cfp
					where	(cfp.org_id = customer_id_in or cfp.org_id is null)
					)
				and cfm.channel_family_id = cf.id
				and cf.org_id is not null
				and cf.org_id != customer_id_in
			);
	begin
		for one in everything loop
			if one.type in ('U','S') then
				prune_group(one.id, one.type, one.quantity);
			else
				prune_family(customer_id_in, one.id, one.quantity);
			end if;
		end loop;
	end prune_everything;

	procedure subscribe_newest_servers (
		customer_id_in in number
	) is
		-- find servers without base channels
		cursor servers(cid_in in number) is
			select	s.id
			from	rhnServer			s
			where	1=1
				and s.org_id = cid_in
				and not exists (
						select 1
						from	rhnChannel			c,
								rhnServerChannel	sc
						where	sc.server_id = s.id
							and sc.channel_id = c.id
							and c.parent_channel is null
					)
			order by s.modified desc;
		channel_id number;
	begin
		for server in servers(customer_id_in) loop
			channel_id := rhn_channel.guess_server_base(server.id);
			if channel_id is not null then
				begin
					rhn_channel.subscribe_server(server.id, channel_id);
					commit;
				-- exception is really channel_family_no_subscriptions
				exception
					when others then
						null;
				end;
			end if;
		end loop;
	end subscribe_newest_servers;
end rhn_entitlements;
/
show errors

create or replace procedure
create_first_org
(
	name_in in varchar2,
	password_in in varchar2
) is
	ug_type			number;
	group_val		number;
begin
	insert into web_customer (
		id, name, password,
		oracle_customer_id, oracle_customer_number,
		customer_type
	) values (
		1, name_in, password_in,
		1, 1, 'B'
	);

	select rhn_user_group_id_seq.nextval into group_val from dual;

	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_admin';

	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Admins',
		'Organization Admins for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);

	select rhn_user_group_id_seq.nextval into group_val from dual;

	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_applicant';

	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Applicants',
		'Organization Applicants for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);

	-- config admin is special; it gets created in
	-- rhn_entitlements.set_customer_provisioning instead.

	select rhn_user_group_id_seq.nextval into group_val from dual;

	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'channel_admin';

	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Channel Administrator',
		'Channel Administrator for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);

	-- if they need more than 16GB, they'll call us and we'll whip
	-- out a "can be null" patch, which we should do for next
	-- version anyway.  (I thought we did that for this version?)
	insert into rhnOrgQuota(
		org_id, total
	) values (
		1, 1024*1024*1024*16
	);


	insert into rhnOrgState (org_id, state_id)
	select 1, id
	  from rhnOrgAvailableStates
	 where label = 'paid';

        insert into rhnServerGroup
		( id, name, description, max_members, group_type, org_id )
		select rhn_server_group_id_seq.nextval, sgt.name, sgt.name,
			0, sgt.id, 1
		from rhnServerGroupType sgt
		where sgt.label = 'sw_mgr_entitled';
end;
/
show errors

declare
	cursor orgs is
		select	distinct org_id id
		from	rhnOrgEntitlements oe,
			rhnOrgEntitlementType oet
		where	oet.label in ('rhn_provisioning','rhn_nonlinux')
			and oet.id = oe.entitlement_id
			and not exists (
				select	1
				from	rhnUserGroup ug,
					rhnUserGroupType ugt
				where	ugt.label = 'config_admin'
					and ug.org_id = oe.org_id
					and ugt.id = ug.group_type
			);
	cursor ugs(org_id_in in number) is
		select	rhn_user_group_id_seq.nextval id,
			'Configuration Administrator' name,
			'Configuration Administrator for Org  (1)' description,
			NULL max_members, id group_type, org_id_in org_id
		from	rhnUserGroupType
		where	label = 'config_admin';
begin
	for org in orgs loop
		for ug in ugs(org.id) loop
			insert into rhnUserGroup (
				id, name, description,
				max_members, group_type, org_id
			) values (
				ug.id, ug.name, ug.description,
				ug.max_members, ug.group_type, ug.org_id
			);
		end loop;
	end loop;
end;
/
show errors
