set define off;

-- Trigger Changes
DROP TRIGGER RHN_KSTREE_DEL_TRIG;
DROP TRIGGER RHN_CHANNEL_PACKAGE_DEL_TRIG;
DROP TRIGGER RHN_CHANNEL_ERRATA_DEL_TRIG;
DROP TRIGGER RHN_BL_OBS_CHAN_LAST_MOD_TRIG;


CREATE OR REPLACE TRIGGER RHN_ERRATA_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATA
FOR EACH ROW
begin
     if ( :new.last_modified = :old.last_modified ) or
        ( :new.last_modified is null )  then
        :new.last_modified := sysdate;
     end if;
      :new.modified := sysdate;
end rhn_errata_mod_trig;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_KSTREE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNKICKSTARTABLETREE
FOR EACH ROW
begin
     if (:new.last_modified = :old.last_modified) or
        (:new.last_modified is null ) then
             :new.last_modified := sysdate;
     end if;
   :new.modified := sysdate;
end rhn_kstree_mod_trig;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_CHANNEL_ERRATA_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNCHANNELERRATA
FOR EACH ROW
begin
   :new.modified := sysdate;
end rhn_channel_errata_mod_trig;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_CHANNEL_PACKAGE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNCHANNELPACKAGE
FOR EACH ROW
begin
   :new.modified := sysdate;
end rhn_channel_package_mod_trig;
/
SHOW ERRORS;

-- rhnRegTokenEntitlement
CREATE TABLE RHNREGTOKENENTITLEMENT
(
  REG_TOKEN_ID          NUMBER CONSTRAINT RHN_REG_TOK_ENT_RTID_NN NOT NULL
                        constraint rhn_reg_tok_ent_rtid_fk references rhnRegToken(id)
                        on delete cascade,
  SERVER_GROUP_TYPE_ID  NUMBER CONSTRAINT RHN_REG_TOK_ENT_SGTID_NN NOT NULL
                        constraint rhn_reg_tok_ent_sgtid_fk references rhnServerGroupType(id)
                        on delete cascade
)
LOGGING
NOCACHE
NOPARALLEL;

CREATE UNIQUE INDEX RHN_RTE_RTID_SGTID_UQ_IDX ON RHNREGTOKENENTITLEMENT
(REG_TOKEN_ID, SERVER_GROUP_TYPE_ID)
   storage( freelists 16 )
   initrans 32
   NOLOGGING
   NOPARALLEL;

-- Convert the data from rhnRegToken to rhnRegTokenEntitlement
insert into rhnRegTokenEntitlement
(select id, entitlement_level from rhnRegToken);

insert into rhnRegTokenEntitlement rte
(select rte2.reg_token_id, lookup_sg_type ('enterprise_entitled')
   from rhnRegTokenEntitlement rte2
  where rte2.server_group_type_id = lookup_sg_type ('provisioning_entitled')
    and not exists ( select 1
                       from rhnRegTokenEntitlement rte3
                      where rte3.reg_token_id = rte2.reg_token_id
                        and rte3.server_group_type_id =
                                 lookup_sg_type ('enterprise_entitled')) );

alter table rhnRegToken drop ( entitlement_level );

-- rhnServerPackage conversion
ALTER TABLE RHNSERVERPACKAGE ADD (PACKAGE_ARCH_ID  NUMBER);
DROP INDEX RHN_SP_SNE_IDX;

ALTER TABLE RHNSERVERPACKAGE ADD CONSTRAINT RHN_SERVERPACKAGE_PAID_FK FOREIGN KEY (PACKAGE_ARCH_ID)
    REFERENCES RHNPACKAGEARCH (ID);

create index rhn_sp_snep_idx on
        rhnServerPackage(server_id, name_id, evr_id, package_arch_id)
        parallel 6
        storage ( pctincrease 1 freelists 16 )
        initrans 32
        nologging;

-- rhnServerAction indexes
-- create index rhn_ser_act_sid_aid_s_idx
--         on rhnServerAction(server_id, action_id, status)
--         storage( freelists 16 )
--         initrans 32;
--
-- alter table rhnServerAction
-- modify constraint rhn_server_action_sid_aid_uq
-- using index rhn_ser_act_sid_aid_s_idx;
--
-- drop index rhn_server_action_sid_aid_idx;

-- See if the rhnServerAction index changes have already been made
declare
   index_exists number := 0;
begin

   begin
      select 1
      into index_exists
      from all_indexes
      where index_name = 'RHN_SER_ACT_SID_AID_S_IDX';
   exception when no_data_found then
      index_exists := 0;
   end;

   if index_exists = 0 then
      execute immediate '
             create index rhn_ser_act_sid_aid_s_idx
               on rhnServerAction(server_id, action_id, status)
               storage( freelists 16 )
               initrans 32';
      execute immediate '
             alter table rhnServerAction
             modify constraint rhn_server_action_sid_aid_uq
             using index rhn_ser_act_sid_aid_s_idx';
      execute immediate '
             drop index rhn_server_action_sid_aid_idx';
   end if;
end;
/
show errors;


DROP INDEX RHN_SERVER_ACTION_AID_SID_IDX;

create index rhn_ser_act_aid_sid_s_idx
   on rhnServerAction(action_id, server_id, status)
   storage( freelists 16 )
   initrans 32
   nologging;


-- Entitlement schema

CREATE TABLE RHNFEATURE
(
  ID        NUMBER
            constraint rhn_feature_id primary key,
  LABEL     VARCHAR2(32 BYTE) CONSTRAINT RHN_FEATURE_LABEL_NN NOT NULL,
  NAME      VARCHAR2(64 BYTE) CONSTRAINT RHN_FEATURE_NAME_NN NOT NULL,
  CREATED   DATE                                DEFAULT (sysdate) CONSTRAINT RHN_FEATURE_CREATED_NN NOT NULL,
  MODIFIED  DATE                                DEFAULT (sysdate) CONSTRAINT RHN_FEATURE_MODIFIED_NN NOT NULL
)
LOGGING
NOCACHE;

create unique index rhn_feature_label_uq_idx
        on rhnFeature(label)
        storage ( freelists 16 )
        initrans 32;

CREATE SEQUENCE RHN_FEATURE_SEQ;

ALTER TABLE RHNSERVERGROUPTYPE ADD (PERMANENT  CHAR(1 BYTE) DEFAULT ('Y') CONSTRAINT RHN_SERVERGROUPTYPE_PERM_NN NOT NULL
                                    constraint rhn_servergrouptype_perm_ck check (permanent in ('Y','N')));
ALTER TABLE RHNSERVERGROUPTYPE ADD (IS_BASE  CHAR(1 BYTE) DEFAULT ('Y') CONSTRAINT RHN_SERVERGROUPTYPE_ISBASE_NN NOT NULL
                                    constraint rhn_servergrouptype_isbase_ck check (is_base in ('Y','N')));

create table
rhnServerGroupTypeFeature
(
        server_group_type_id     number
                                 constraint rhn_sgt_sgtid_nn not null
                                 constraint rhn_sgt_sgid_fk references rhnServerGroupType(id),
        feature_id               number
                                 constraint rhn_sgt_fid_nn not null
                                 constraint rhn_sgt_fid_fk references rhnFeature(id),
        created                  date default(sysdate)
                                 constraint rhn_sgt_feat_created_nn not null,
        modified                 date default(sysdate)
                                 constraint rhn_sgt_feat_mod_nn not null
)
        storage ( freelists 16 )
        initrans 32;

create unique index rhn_sgt_feat_sgtid_fid_uq_idx
        on rhnServerGroupTypeFeature(server_group_type_id, feature_id)
        storage ( freelists 16 )
        initrans 32;

create table
rhnSGTypeBaseAddonCompat
(
   base_id     number
               constraint rhn_sgt_bac_bid_nn not null
               constraint rhn_sgt_bac_bid_fk references rhnServerGroupType(id),
   addon_id    number
               constraint rhn_sgt_bac_aid_nn not null
               constraint rhn_sgt_bac_aid_fk references rhnServerGroupType(id)
)
   storage ( freelists 16 )
   initrans 32;


-- Solaris schema

CREATE SEQUENCE RHN_SOLARIS_PS_SEQ;

CREATE TABLE RHNSOLARISPATCHTYPE
(
  ID     NUMBER
         constraint rhn_solaris_pt_pk primary key,
  NAME   VARCHAR2(32 BYTE) CONSTRAINT RHN_SOLARIS_PT_NAME_NN NOT NULL,
  LABEL  VARCHAR2(32 BYTE) CONSTRAINT RHN_SOLARIS_PT_LABEL_NN NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

CREATE SEQUENCE RHN_SOLARIS_PT_SEQ;

create table rhnSolarisPackage (
   package_id              number
                           constraint rhn_solaris_pkg_pid_pk primary key
                           constraint rhn_solaris_pkg_pid_fk references rhnPackage(id)
                           on delete cascade,
   category                varchar2(2048)
                           constraint rhn_solaris_pkg_cat_nn not null,
   pkginfo                 varchar2(4000),
   pkgmap                  blob,
   intonly                 char(1) default 'N'
                           constraint rhn_solaris_pkg_io_ck check ( intonly in ('Y','N'))
)
storage( pctincrease 1 freelists 16 )
initrans 32;

create table rhnSolarisPatch (
   package_id        number
                     constraint rhn_solaris_p_pk primary key
                     constraint rhn_solaris_p_fk references rhnPackage(id)
                     on delete cascade,
   solaris_release   varchar2(64),
   sunos_release     varchar2(64),
   patch_type        number
                     constraint rhn_solaris_p_pt_nn not null
                     constraint rhn_solaris_p_pt_fk references rhnSolarisPatchType (id)
                     on delete set null,
   created           date default(sysdate)
                     constraint rhn_solaris_p_created_nn not null,
   modified          date default(sysdate)
                     constraint rhn_solaris_p_modified_nn not null,
   readme            blob
                     constraint rhn_solaris_p_rdme_nn not null,
   patchinfo         varchar2(4000)
)
storage( pctincrease 1 freelists 16 )
initrans 32;

create table rhnSolarisPatchPackages (
   patch_id             number
                        constraint rhn_solaris_pp_nn not null
                        constraint rhn_solaris_pp_fk references rhnPackage(id)
                        on delete cascade,
   package_nevra_id     number
                        constraint rhn_solaris_pnid_nn not null
                        constraint rhn_solaris_pnid_fk references rhnPackageNEVRA(id)
                        on delete cascade
)
storage( pctincrease 1 freelists 16 )
initrans 32;

CREATE INDEX RHN_SOLARIS_PP_PID_PNID_IDX ON RHNSOLARISPATCHPACKAGES
(PATCH_ID, PACKAGE_NEVRA_ID)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SOLARIS_PP_PNID_PID_IDX ON RHNSOLARISPATCHPACKAGES
(PACKAGE_NEVRA_ID, PATCH_ID)
LOGGING
NOPARALLEL;

create table rhnSolarisPatchSet (
   package_id        number
                     constraint rhn_solaris_ps_pid_pk primary key
                     constraint rhn_solaris_ps_pid_fk references rhnPackage(id)
                     on delete cascade,
   readme            blob,
   set_date          date default(sysdate)
                     constraint rhn_solaris_ps_sd_nn not null,
   created           date default(sysdate)
                     constraint rhn_solaris_ps_created_nn not null,
   modified          date default(sysdate)
                     constraint rhn_solaris_ps_modified_nn not null
)
storage( pctincrease 1 freelists 16 )
initrans 32;

create table rhnSolarisPatchSetMembers (
   patch_id          number
                     constraint rhn_solaris_psm_pid_nn not null
                     constraint rhn_solaris_psm_pid_fk references rhnPackage(id)
                     on delete cascade,
   patch_set_id      number
                     constraint rhn_solaris_psm_psid_nn not null
                     constraint rhn_solaris_psm_psid_fk references rhnPackage(id)
                     on delete cascade,
   patch_order       number,
   created           date default(sysdate)
                     constraint rhn_solaris_psm_created_nn not null,
   modified          date default(sysdate)
                     constraint rhn_solaris_psm_modified_nn not null
)
storage( pctincrease 1 freelists 16 )
initrans 32;

CREATE INDEX RHN_SOLARIS_PSM_PID_PSID_IDX ON RHNSOLARISPATCHSETMEMBERS
(PATCH_ID, PATCH_SET_ID)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SOLARIS_PSM_PSID_PID_IDX ON RHNSOLARISPATCHSETMEMBERS
(PATCH_SET_ID, PATCH_ID)
LOGGING
NOPARALLEL;

CREATE OR REPLACE TRIGGER RHN_SOLARIS_PSM_MOD_TRIG
BEFORE UPDATE ON RHNSOLARISPATCHSETMEMBERS
FOR EACH ROW
begin
   :new.modified := sysdate;
end;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_SOLARIS_PS_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNSOLARISPATCHSET
FOR EACH ROW
begin
   :new.modified := sysdate;
end;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_SOLARIS_P_MOD_TRIG
BEFORE UPDATE ON RHNSOLARISPATCH
FOR EACH ROW
begin
   :new.modified := sysdate;
end;
/
SHOW ERRORS;


create table rhnActionPackageOrder (
   action_package_id       number
                           constraint rhn_act_pkg_apid_nn not null
                           constraint rhn_act_pkg_apid_fk references rhnActionPackage(id)
                           on delete cascade,
   package_order           number
                           constraint rhn_act_pkg_order_nn not null
)
storage( pctincrease 1 freelists 16 )
initrans 32;

create table rhnSolarisPatchedPackage (
   server_id            number
                        constraint rhn_solaris_patchedp_sid_nn not null
                        constraint rhn_solaris_patchedp_sid_fk references rhnServer(id)
                        on delete cascade,
   patch_id             number
                        constraint rhn_solaris_patchedp_pid_nn not null
                        constraint rhn_solaris_patchedp_pid_fk references rhnPackage(id)
                        on delete cascade,
   package_nevra_id     number
                        constraint rhn_solaris_patchedp_pnid_nn not null
                        constraint rhn_solaris_patchedp_pnid_fk references rhnPackageNEVRA(id)
                        on delete cascade
)
storage( pctincrease 1 freelists 16 )
initrans 32;

CREATE INDEX RHN_ACT_PKG_APID_IDX ON RHNACTIONPACKAGEORDER
(ACTION_PACKAGE_ID)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SOLARIS_PATCHEDP_SID_IDX ON RHNSOLARISPATCHEDPACKAGE
(SERVER_ID)
NOLOGGING
NOPARALLEL;

-- User Disablement schema

CREATE SEQUENCE RHN_WCON_DISABLED_SEQ;

create table rhnWebContactChangeState (
    id                     number
                           constraint rhn_cont_change_state_id_pk primary key,
    label                  varchar2(32)
                           constraint rhn_cont_change_state_nn not null
)
storage( pctincrease 1 freelists 16 )
initrans 32;

CREATE SEQUENCE RHN_WCON_CHANGE_STATE_SEQ;

create table rhnWebContactChangeLog (
    id                     number
                           constraint rhn_wcon_cl_id_pk primary key,
    web_contact_id         number
                           constraint rhn_wcon_cl_wcon_id_nn not null
                           constraint rhn_wcon_cl_wcon_id_fk references web_contact(id)
                           on delete cascade,
    web_contact_from_id    number
                           constraint rhn_wcon_cl_wcon_from_id_fk references web_contact(id)
                           on delete set null,
    change_state_id        number
                           constraint rhn_wcon_cl_change_sid_nn not null
                           constraint rhn_wcon_cl_csid_fk references rhnWebContactChangeState(id),
    date_completed         date default(sysdate)
                           constraint rhn_wcon_cl_modified_nn not null
)
storage( pctincrease 1 freelists 16 )
initrans 32;

CREATE INDEX RHN_WCON_DISABLED_WCON_ID_IDX ON RHNWEBCONTACTCHANGELOG
(WEB_CONTACT_ID)
NOLOGGING
NOPARALLEL;

-- Miscellaneous Changes
CREATE UNIQUE INDEX RHN_PKGNEVRA_NID_EID_PAID_UQ ON RHNPACKAGENEVRA
(NAME_ID, EVR_ID, PACKAGE_ARCH_ID)
LOGGING
NOPARALLEL;

ALTER TABLE RHNERRATA
MODIFY(LAST_MODIFIED  NOT NULL);

ALTER TABLE RHNERRATATMP ADD (LAST_MODIFIED  DATE DEFAULT (sysdate) CONSTRAINT RHN_ERRATATMP_LAST_MOD_NN NOT NULL);

CREATE OR REPLACE TRIGGER RHN_ERRATATMP_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATATMP
FOR EACH ROW
begin
        :new.modified := sysdate;
        :new.last_modified := sysdate;
end;
/
SHOW ERRORS;

CREATE UNIQUE INDEX RHN_CONFCHAN_OID_LABEL_NAME_UQ ON RHNCONFIGCHANNEL
(ORG_ID, LABEL, NAME)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SNMP_ALRT_SCID_IDX ON RHN_SNMP_ALERT
(SENDER_CLUSTER_ID)
NOLOGGING
NOPARALLEL;

ALTER TABLE RHN_REDIRECTS ADD (RECURRING  NUMBER(12) DEFAULT 0 CONSTRAINT RHN_RDRCT_RECURRING_NN NOT NULL
                               CONSTRAINT RHN_RDRCT_RECUR_VALID CHECK (recurring in (0, 1)));
ALTER TABLE RHN_REDIRECTS ADD (RECURRING_FREQUENCY  NUMBER(12) DEFAULT 2
                               CONSTRAINT RHN_RDRCT_RECUR_FREQ_VALID CHECK (recurring_frequency in (2,3,6)));
ALTER TABLE RHN_REDIRECTS ADD (RECURRING_DURATION  NUMBER(12) DEFAULT 0);
ALTER TABLE RHN_REDIRECTS ADD (RECURRING_DUR_TYPE  NUMBER(12) DEFAULT 12
                               CONSTRAINT RHN_RDRCT_REC_DTYPE_VALID CHECK ( recurring_dur_type in (12,11,5,3,1) ) );

CREATE INDEX RHN_CMETH_SNDR_SCID_IDX ON RHN_CONTACT_METHODS
(SENDER_SAT_CLUSTER_ID)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_LL_NTSNTS_NSID_IDX ON RHN_LL_NETSAINT
(NETSAINT_ID)
NOLOGGING
NOPARALLEL;

CREATE INDEX RHN_RDRME_CMID_IDX ON RHN_REDIRECT_METHOD_TARGETS
(CONTACT_METHOD_ID)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SAT_NODE_SCID_IDX ON RHN_SAT_NODE
(SAT_CLUSTER_ID)
NOLOGGING
NOPARALLEL;

CREATE INDEX RHN_SRVR_MSSG_SRVR_EVNT_IDX ON RHNSERVERMESSAGE
(SERVER_EVENT)
LOGGING
NOPARALLEL;

CREATE INDEX RHN_SRVR_TKN_RGS_SID_TID_IDX ON RHNSERVERTOKENREGS
(SERVER_ID, TOKEN_ID)
NOLOGGING
NOPARALLEL;

CREATE INDEX RHN_ACT_KEY_KSSID_RTID_IDX ON RHNACTIVATIONKEY
(KS_SESSION_ID, REG_TOKEN_ID)
NOLOGGING
NOPARALLEL;

CREATE UNIQUE INDEX RHN_ACTIONCF_NAME_SAC_UQ ON RHNACTIONCONFIGFILENAME
(SERVER_ID, ACTION_ID, CONFIG_FILE_NAME_ID)
LOGGING
NOPARALLEL;

ALTER TABLE RHNPACKAGENEVRA ADD CONSTRAINT RHN_PKGNEVRA_NID_EID_PAID_UQ UNIQUE (NAME_ID, EVR_ID, PACKAGE_ARCH_ID);
ALTER TABLE RHNCONFIGCHANNEL ADD CONSTRAINT RHN_CONFCHAN_OID_LABEL_NAME_UQ UNIQUE (ORG_ID, LABEL, NAME);
ALTER TABLE RHNERRATA ADD CONSTRAINT RHN_ERRATA_LAST_MOD_NN CHECK ("LAST_MODIFIED" IS NOT NULL);

CREATE OR REPLACE FORCE VIEW RHNSERVERENTITLEMENTVIEW
(SERVER_ID, SERVER_GROUP_TYPE_ID, LABEL, PERMANENT, IS_BASE)
AS
select
   distinct
   sgm.server_id,
   sgt.id,
   sgt.label,
   sgt.permanent,
   sgt.is_base
from
   rhnServerGroupType sgt,
   rhnServerGroup sg,
   rhnServerGroupMembers sgm
where
   sg.id = sgm.server_group_id
   and sg.group_type = sgt.id;

-- Package Specs
CREATE OR REPLACE package
rhn_cache
is
   version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
   procedure update_perms_for_server(
      server_id_in in number
   );
   procedure update_perms_for_user(
      user_id_in in number
   );
   procedure update_perms_for_server_group(
      server_group_id_in in number
   );
end rhn_cache;
/
SHOW ERRORS;

CREATE OR REPLACE package rhn_user
is
   version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
    function check_role(user_id_in in number, role_in in varchar2) return number;
    PRAGMA RESTRICT_REFERENCES(check_role, WNDS, RNPS, WNPS);
    function check_role_implied(user_id_in in number, role_in in varchar2) return number;
    PRAGMA RESTRICT_REFERENCES(check_role_implied, WNDS, RNPS, WNPS);
    function get_org_id(user_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_id, WNDS, RNPS, WNPS);
   function find_mailable_address(user_id_in in number) return varchar2;
   procedure add_servergroup_perm(
      user_id_in in number,
      server_group_id_in in number
   );
   procedure remove_servergroup_perm(
      user_id_in in number,
      server_group_id_in in number
   );
   procedure add_to_usergroup(
      user_id_in in number,
      user_group_id_in in number
   );
   procedure add_users_to_usergroups(
      user_id_in in number
   );
   procedure remove_from_usergroup(
      user_id_in in number,
      user_group_id_in in number
   );
   procedure remove_users_from_servergroups(
      user_id_in in number
   );
end rhn_user;
/
SHOW ERRORS;

CREATE OR REPLACE PACKAGE rhn_channel
IS
   version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
    CURSOR server_base_subscriptions(server_id_in NUMBER) IS
         SELECT C.id
        FROM rhnChannel C, rhnServerChannel SC
       WHERE C.id = SC.channel_id
         AND SC.server_id = server_id_in
         AND C.parent_channel IS NULL;
    CURSOR check_server_subscription(server_id_in NUMBER, channel_id_in NUMBER) IS
           SELECT channel_id
        FROM rhnServerChannel
       WHERE server_id = server_id_in
         AND channel_id = channel_id_in;
    CURSOR check_server_parent_membership(server_id_in NUMBER, channel_id_in NUMBER) IS
         SELECT C.id
        FROM rhnChannel C, rhnServerChannel SC
       WHERE C.parent_channel = channel_id_in
         AND C.id = SC.channel_id
         AND SC.server_id = server_id_in;
    CURSOR channel_family_perm_cursor(channel_family_id_in NUMBER, org_id_in NUMBER) IS
           SELECT *
        FROM rhnOrgChannelFamilyPermissions
       WHERE channel_family_id = channel_family_id_in
         AND org_id = org_id_in;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER);
    FUNCTION get_license_path(channel_id_in IN NUMBER) RETURN VARCHAR2;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0,
                                 deleting_server in number := 0 );
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in number := null);
    FUNCTION guess_server_base(server_id_in IN NUMBER) RETURN NUMBER;
    FUNCTION base_channel_for_release_arch(release_in in varchar2,
   server_arch_in in varchar2, org_id_in in number := -1,
   user_id_in in number := null) RETURN number;
    FUNCTION base_channel_rel_archid(release_in in varchar2,
   server_arch_id_in in number, org_id_in in number := -1,
   user_id_in in number := null) RETURN number;
    FUNCTION channel_priority(channel_id_in in number) RETURN number;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    procedure bulk_server_basechange_from(
   set_label_in in varchar2,
   set_uid_in in number,
   old_channel_id_in in number,
   new_channel_id_in in number);
    procedure bulk_guess_server_base(
   set_label_in in varchar2,
   set_uid_in in number);
    procedure bulk_guess_server_base_from(
   set_label_in in varchar2,
   set_uid_in in number,
   channel_id_in in number);
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER, deleting_server in number := 0 );
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER);
    FUNCTION family_for_channel(channel_id_in IN NUMBER) RETURN NUMBER;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;
    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number);
    procedure delete_server_channels(server_id_in in number);
    procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)');
    function get_org_id(channel_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_id, WNDS, RNPS, WNPS);
    function get_org_access(channel_id_in in number, org_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_access, WNDS, RNPS, WNPS);
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number) return number;
    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
      RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check_debug, WNDS, RNPS, WNPS);
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
      RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check, WNDS, RNPS, WNPS);
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
      RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(loose_user_role_check, WNDS, RNPS, WNPS);
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
      RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(direct_user_role_check, WNDS, RNPS, WNPS);
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
      RETURN NUMBER;
    PROCEDURE update_channel ( channel_id_in in number, invalidate_ss in number := 0,
                               date_to_use in date := sysdate );
    PROCEDURE  update_channels_by_package ( package_id_in in number, date_to_use in date := sysdate );
     PROCEDURE update_channels_by_errata ( errata_id_in number, date_to_use in date := sysdate );
    PRAGMA RESTRICT_REFERENCES(org_channel_setting, WNDS, RNPS, WNPS);
END rhn_channel;
/
SHOW ERRORS;

CREATE OR REPLACE package rhn_entitlements
is
   body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
   type ents_array is varray(10) of rhnServerGroupType.label%TYPE;
    function entitlement_grants_service (
       entitlement_in in varchar2,
      service_level_in in varchar2
   ) return number;
   function lookup_entitlement_group (
      org_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) return number;
   function create_entitlement_group (
      org_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) return number;
   function can_entitle_server (
      server_id_in   in number,
      type_label_in  in varchar2
   )
   return number;
   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   )
   return number;
   function find_compatible_sg (
      server_id_in in number,
      type_label_in in varchar2,
      sgid_out out number
   )
   return boolean;
   procedure entitle_server (
      server_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   );
   procedure remove_server_entitlement (
      server_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   );
   procedure unentitle_server (
      server_id_in in number
   );
   function get_server_entitlement (
      server_id_in in number
   ) return ents_array;
   procedure modify_org_service (
      org_id_in in number,
      service_label_in in varchar2,
      enable_in in char
   );
    procedure set_customer_enterprise (
      customer_id_in in number
   );
   procedure set_customer_provisioning (
      customer_id_in in number
   );
   procedure set_customer_nonlinux (
      customer_id_in in number
   );
    procedure unset_customer_enterprise (
      customer_id_in in number
   );
   procedure unset_customer_provisioning (
      customer_id_in in number
   );
   procedure unset_customer_nonlinux (
      customer_id_in in number
   );
    procedure set_group_count (
      customer_id_in in number,  -- customer_id
      type_in in char,        -- 'U' or 'S'
      group_type_in in number,   -- rhn[User|Server]GroupType.id
      quantity_in in number      -- quantity
    );
    procedure set_family_count (
      customer_id_in in number,     -- customer_id
      channel_family_id_in in number,  -- 246
      quantity_in in number         -- 3
    );
    procedure entitle_last_modified_servers (
      customer_id_in in number,  -- customer_id
      type_label_in in varchar2, -- 'enterprise_entitled'
      quantity_in in number      -- 3
    );
   procedure prune_everything (
      customer_id_in in number
   );
   procedure subscribe_newest_servers (
      customer_id_in in number
   );
end rhn_entitlements;
/
SHOW ERRORS;

CREATE OR REPLACE PACKAGE rhn_org
IS
   version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
    CURSOR server_group_by_label(org_id_in NUMBER, group_label_in VARCHAR2) IS
         SELECT SG.*
        FROM rhnServerGroupType SGT,
           rhnServerGroup SG
       WHERE SG.group_type = SGT.id
         AND SGT.label = group_label_in
         AND SG.org_id = org_id_in;
    FUNCTION find_server_group_by_type(org_id_in NUMBER, group_label_in VARCHAR2) RETURN NUMBER;
   procedure delete_user(user_id_in number);
END rhn_org;
/
SHOW ERRORS;

-- Package bodies
CREATE OR REPLACE package body
rhn_cache
is
   body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
   procedure update_perms_for_server(
      server_id_in in number
   ) is
   begin
      delete from rhnUserServerPerms where server_id = server_id_in;
      insert into rhnUserServerPerms(user_id, server_id) (
            select   distinct user_id, server_id_in
            from  rhnUserServerPermsDupes
            where server_id = server_id_in
         );
   end update_perms_for_server;
   procedure update_perms_for_user(
      user_id_in in number
   ) is
   begin
      delete from rhnUserServerPerms where user_id = user_id_in;
      insert into rhnUserServerPerms(user_id, server_id) (
            select   distinct user_id_in, server_id
            from  rhnUserServerPermsDupes
            where user_id = user_id_in
         );
   end update_perms_for_user;
   procedure update_perms_for_server_group(
      server_group_id_in in number
   ) is
      cursor users is
         select   usgp.user_id id
         from  rhnUserServerGroupPerms usgp
         where usgp.server_group_id = server_group_id_in
            and not exists (
               select   1
               from  rhnUserGroup ug,
                     rhnUserGroupMembers ugm,
                     rhnServerGroup sg,
                     rhnUserGroupType ugt
               where ugt.label = 'org_admin'
                  and sg.id = server_group_id_in
                  and ugm.user_id = usgp.user_id
                  and ug.org_id = sg.org_id
                  and ugm.user_group_id = ug.id
               );
   begin
      for u in users loop
         update_perms_for_user(u.id);
      end loop;
   end update_perms_for_server_group;
end rhn_cache;
/
SHOW ERRORS;

CREATE OR REPLACE PACKAGE BODY rhn_channel
IS
   body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
   cursor   base_channel_cursor(
      release_in in varchar2,
      server_arch_id_in in number,
      org_id_in in number
   ) return rhnChannel%ROWTYPE is
      select distinct c.*
      from  rhnDistChannelMap       dcm,
            rhnServerChannelArchCompat scac,
            rhnChannel              c,
            rhnChannelPermissions      cp
      where cp.org_id = org_id_in
         and cp.channel_id = c.id
         and c.parent_channel is null
         and c.id = dcm.channel_id
         and c.channel_arch_id = dcm.channel_arch_id
         and dcm.release = release_in
         and scac.server_arch_id = server_arch_id_in
         and scac.channel_arch_id = c.channel_arch_id;
    FUNCTION get_license_path(channel_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
      license_val VARCHAR2(1000);
    BEGIN
      SELECT CFL.license_path INTO license_val
     FROM rhnChannelFamilyLicense CFL, rhnChannelFamilyMembers CFM
    WHERE CFM.channel_id = channel_id_in
      AND CFM.channel_family_id = CFL.channel_family_id;
      RETURN license_val;
    EXCEPTION
      WHEN NO_DATA_FOUND
       THEN
       RETURN NULL;
    END get_license_path;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER)
    IS
      channel_family_id_val NUMBER;
    BEGIN
      channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
   IF channel_family_id_val IS NULL
   THEN
       rhn_exception.raise_exception('channel_subscribe_no_family');
   END IF;
   IF rhn_channel.get_license_path(channel_id_in) IS NULL
   THEN
       rhn_exception.raise_exception('channel_consent_no_license');
   END IF;
   INSERT INTO rhnChannelFamilyLicenseConsent (channel_family_id, user_id, server_id)
   VALUES (channel_family_id_val, user_id_in, server_id_in);
    END license_consent;
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in in number := null)
    IS
        channel_parent_val      rhnChannel.parent_channel%TYPE;
        parent_subscribed       BOOLEAN;
        server_has_base_chan    BOOLEAN;
   server_already_in_chan  BOOLEAN;
   channel_family_id_val   NUMBER;
   server_org_id_val       NUMBER;
   available_subscriptions NUMBER;
   consenting_user         NUMBER;
   allowed        number := 0;
    BEGIN
      if user_id_in is not null then
      allowed := rhn_channel.user_role_check(channel_id_in, user_id_in, 'subscribe');
   else
      allowed := 1;
   end if;
   if allowed = 0 then
      rhn_exception.raise_exception('no_subscribe_permissions');
   end if;
        SELECT parent_channel INTO channel_parent_val FROM rhnChannel WHERE id = channel_id_in;
        IF channel_parent_val IS NOT NULL
        THEN
            parent_subscribed := FALSE;
            FOR check_subscription IN check_server_subscription(server_id_in, channel_parent_val)
            LOOP
                parent_subscribed := TRUE;
            END LOOP check_subscription;
            IF NOT parent_subscribed
            THEN
            RETURN;
            END IF;
        ELSE
            server_has_base_chan := FALSE;
            FOR base IN server_base_subscriptions(server_id_in)
            LOOP
                server_has_base_chan := TRUE;
            END LOOP base;
            IF server_has_base_chan
            THEN
                rhn_exception.raise_exception('channel_server_one_base');
            END IF;
        END IF;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF server_already_in_chan
        THEN
          RETURN;
        END IF;
   channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
   IF channel_family_id_val IS NULL
   THEN
       rhn_exception.raise_exception('channel_subscribe_no_family');
   END IF;
   SELECT org_id INTO server_org_id_val
     FROM rhnServer
    WHERE id = server_id_in;
   available_subscriptions := rhn_channel.available_family_subscriptions(channel_family_id_val, server_org_id_val);
   IF available_subscriptions IS NULL OR available_subscriptions > 0
   THEN
       IF rhn_channel.get_license_path(channel_id_in) IS NOT NULL
       THEN
         BEGIN
         SELECT user_id INTO consenting_user
        FROM rhnChannelFamilyLicenseConsent
       WHERE channel_family_id = channel_family_id_val
         AND server_id = server_id_in;
      EXCEPTION
                WHEN NO_DATA_FOUND
            THEN
             rhn_exception.raise_exception('channel_subscribe_no_consent');
      END;
       END IF;
       insert into rhnServerHistory (id,server_id,summary,details) (
         select   rhn_event_id_seq.nextval,
         server_id_in,
         'subscribed to channel ' || SUBSTR(c.label, 0, 106),
         c.label
      from  rhnChannel c
      where c.id = channel_id_in
       );
       UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
            INSERT INTO rhnServerChannel (server_id, channel_id) VALUES (server_id_in, channel_id_in);
          rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
          queue_server(server_id_in, immediate_in);
   ELSE
       rhn_exception.raise_exception('channel_family_no_subscriptions');
   END IF;
    END subscribe_server;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
        END LOOP server;
    END bulk_subscribe_server;
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
       IF rhn_server.can_change_base_channel(server.element) = 1
       THEN
                rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
                rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
            END IF;
        END LOOP server;
    END bulk_server_base_change;
    procedure bulk_server_basechange_from(
        set_label_in in varchar2,
        set_uid_in in number,
        old_channel_id_in in number,
        new_channel_id_in in number
    ) is
    cursor servers is
        select  sc.server_id id
        from    rhnChannel nc,
                rhnServerChannelArchCompat scac,
                rhnServer s,
                rhnChannel oc,
                rhnServerChannel sc,
                rhnSet st
        where   1=1
            and st.label = set_label_in
            and st.user_id = set_uid_in
            and st.element = sc.server_id
            and sc.channel_id = old_channel_id_in
            and sc.channel_id = oc.id
            and oc.parent_channel is null
            and nc.id = new_channel_id_in
            and nc.parent_channel is null
            and sc.server_id = s.id
            and s.server_arch_id = scac.server_arch_id
            and scac.channel_arch_id = nc.channel_arch_id;
    begin
        for s in servers loop
            insert into rhnSet (
                    user_id, label, element
                ) values (
                    set_uid_in,
                    set_label_in || 'basechange',
                    s.id
                );
        end loop channel;
        bulk_server_base_change(new_channel_id_in,
                                set_label_in || 'basechange',
                                set_uid_in);
        delete from rhnSet
            where   label = set_label_in||'basechange'
                and user_id = set_uid_in;
    end bulk_server_basechange_from;
    procedure bulk_guess_server_base(
   set_label_in in varchar2,
   set_uid_in in number
    ) is
   channel_id number;
    begin
   for server in rhn_set.set_iterator(set_label_in, set_uid_in)
   loop
       begin
         if rhn_server.can_change_base_channel(server.element) = 1
      then
                  channel_id := guess_server_base(TO_NUMBER(server.element));
             rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
             rhn_channel.subscribe_server(TO_NUMBER(server.element), channel_id, 0, set_uid_in);
      end if;
       exception when others then
         null;
       end;
   end loop server;
    end;
    function guess_server_base(
      server_id_in in number
    ) RETURN number is
   cursor server_cursor is
       select s.server_arch_id, s.release, s.org_id
         from rhnServer s
        where s.id = server_id_in;
    begin
   for s in server_cursor loop
       for channel in base_channel_cursor(s.release,
      s.server_arch_id, s.org_id)
       loop
      return channel.id;
       end loop base_channel_cursor;
        end loop server_cursor;
        return null;
    end;
    function normalize_server_arch(server_arch_in in varchar2)
    return varchar2
    is
   suffix VARCHAR2(128) := '-redhat-linux';
   suffix_len NUMBER := length(suffix);
    begin
   if server_arch_in is NULL then
       return NULL;
   end if;
        if instr(server_arch_in, '-') > 0
   then
       return server_arch_in;
   end if;
   return server_arch_in || suffix;
    end normalize_server_arch;
    function base_channel_for_release_arch(
   release_in in varchar2,
   server_arch_in in varchar2,
   org_id_in in number := -1,
   user_id_in in number := null
    ) return number is
   server_arch varchar2(256) := normalize_server_arch(server_arch_in);
   server_arch_id number;
    begin
   begin
       select id
         into server_arch_id
         from rhnServerArch
        where label = server_arch;
   exception
       when no_data_found then
      rhn_exception.raise_exception('server_arch_not_found');
   end;
   return base_channel_rel_archid(release_in, server_arch_id,
       org_id_in, user_id_in);
    end base_channel_for_release_arch;
    function base_channel_rel_archid(
   release_in in varchar2,
   server_arch_id_in in number,
   org_id_in in number := -1,
   user_id_in in number := null
    ) return number is
   denied_channel_id number := null;
   valid_org_id number := org_id_in;
   valid_user_id number := user_id_in;
   channel_subscribable number;
    begin
   if org_id_in = -1 and user_id_in is not null then
       begin
      select org_id
        into valid_org_id
        from web_contact
       where id = user_id_in;
       exception
      when no_data_found then
          valid_user_id := null;
          valid_org_id := -1;
       end;
   end if;
   for c in base_channel_cursor(release_in, server_arch_id_in, valid_org_id)
   loop
       if valid_user_id is null then
      return c.id;
       end if;
       select loose_user_role_check(c.id, user_id_in, 'subscribe')
         into channel_subscribable
         from dual;
       if channel_subscribable = 1 then
      return c.id;
       end if;
       denied_channel_id := c.id;
   end loop base_channel_fetch;
   if denied_channel_id is not null then
       rhn_exception.raise_exception('no_subscribe_permissions');
   end if;
   return NULL;
    end base_channel_rel_archid;
    procedure bulk_guess_server_base_from(
   set_label_in in varchar2,
   set_uid_in in number,
   channel_id_in in number
    ) is
   cursor channels(server_id_in in number) is
       select  rsc.channel_id
       from rhnServerChannel rsc,
         rhnChannel rc
       where   server_id_in = rsc.server_id
         and rsc.channel_id = rc.id
         and rc.parent_channel is null;
    begin
   for server in rhn_set.set_iterator(set_label_in, set_uid_in)
   loop
       for channel in channels(server.element)
       loop
      if channel.channel_id = channel_id_in
      then
          insert into rhnSet (user_id, label, element) values (set_uid_in, set_label_in || 'baseguess', server.element);
      end if;
       end loop channel;
   end loop server;
   bulk_guess_server_base(set_label_in||'baseguess',set_uid_in);
   delete from rhnSet where label = set_label_in||'baseguess' and user_id = set_uid_in;
    end;
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER, deleting_server IN NUMBER := 0 )
    IS
      cursor server_channels(server_id_in in number) is
      select   s.org_id, sc.channel_id, cfm.channel_family_id
      from  rhnServer s,
         rhnServerChannel sc,
         rhnChannelFamilyMembers cfm
      where s.id = server_id_in
         and s.id = sc.server_id
         and sc.channel_id = cfm.channel_id;
    BEGIN
      for channel in server_channels(server_id_in)
   loop
      unsubscribe_server(server_id_in, channel.channel_id, 1, 1, deleting_server);
      rhn_channel.update_family_counts(channel.channel_family_id, channel.org_id);
   end loop channel;
    END clear_subscriptions;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0,
                                 deleting_server IN NUMBER := 0 )
    IS
   channel_family_id_val   NUMBER;
   server_org_id_val       NUMBER;
   available_subscriptions NUMBER;
   server_already_in_chan  BOOLEAN;
   cursor   channel_family_is_proxy(channel_family_id_in in number) is
      select   1
      from  rhnChannelFamily
      where id = channel_family_id_in
          and label = 'rhn-proxy';
   cursor   channel_family_is_satellite(channel_family_id_in in number) is
      select   1
      from  rhnChannelFamily
      where id = channel_family_id_in
          and label = 'rhn-satellite';
   cursor local_chk_server_parent_memb (
         server_id_in number,
         channel_id_in number ) is
      select   c.id
      from  rhnChannel        c,
            rhnServerChannel  sc
      where 1=1
         and c.parent_channel = channel_id_in
         and c.id = sc.channel_id
         and sc.server_id = server_id_in;
    BEGIN
        FOR child IN local_chk_server_parent_memb(server_id_in, channel_id_in)
        LOOP
       if unsubscribe_children_in = 1 then
         unsubscribe_server(server_id_in => server_id_in,
                        channel_id_in => child.id,
                        immediate_in => immediate_in,
                        unsubscribe_children_in => unsubscribe_children_in,
                        deleting_server => deleting_server);
       else
         rhn_exception.raise_exception('channel_unsubscribe_child_exists');
       end if;
        END LOOP child;
   server_already_in_chan := FALSE;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF NOT server_already_in_chan
        THEN
          RETURN;
        END IF;
   if deleting_server = 0 then
      insert into rhnServerHistory (id,server_id,summary,details) (
          select  rhn_event_id_seq.nextval,
                server_id_in,
             'unsubscribed from channel ' || SUBSTR(c.label, 0, 106),
             c.label
          from    rhnChannel c
          where   c.id = channel_id_in
      );
      UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
   end if;
   DELETE FROM rhnServerChannel WHERE server_id = server_id_in AND channel_id = channel_id_in;
   if deleting_server = 0 then
      queue_server(server_id_in, immediate_in);
   end if;
   channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
   IF channel_family_id_val IS NULL
   THEN
       rhn_exception.raise_exception('channel_unsubscribe_no_family');
   END IF;
   for ignore in channel_family_is_satellite(channel_family_id_val) loop
      delete from rhnSatelliteInfo where server_id = server_id_in;
      delete from rhnSatelliteChannelFamily where server_id = server_id_in;
   end loop;
   for ignore in channel_family_is_proxy(channel_family_id_val) loop
      delete from rhnProxyInfo where server_id = server_id_in;
   end loop;
   DELETE FROM rhnChannelFamilyLicenseConsent
    WHERE channel_family_id = channel_family_id_val
      AND server_id = server_id_in;
   SELECT org_id INTO server_org_id_val
     FROM rhnServer
    WHERE id = server_id_in;
   rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    END unsubscribe_server;
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.unsubscribe_server(server.element, channel_id_in, 0);
        END LOOP server;
    END bulk_unsubscribe_server;
    FUNCTION family_for_channel(channel_id_in IN NUMBER)
    RETURN NUMBER
    IS
      channel_family_id_val NUMBER;
    BEGIN
      SELECT channel_family_id INTO channel_family_id_val
     FROM rhnChannelFamilyMembers
         WHERE channel_id = channel_id_in;
   RETURN channel_family_id_val;
    EXCEPTION
        WHEN NO_DATA_FOUND
   THEN
       RETURN NULL;
    END family_for_channel;
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
      cfp channel_family_perm_cursor%ROWTYPE;
      current_members_val NUMBER;
   max_members_val     NUMBER;
   found               NUMBER;
    BEGIN
   IF NOT channel_family_perm_cursor%ISOPEN
   THEN
       OPEN channel_family_perm_cursor(channel_family_id_in, org_id_in);
   END IF;
   FETCH channel_family_perm_cursor INTO cfp;
   WHILE channel_family_perm_cursor%FOUND
      LOOP
       found := 1;
       current_members_val := cfp.current_members;
       max_members_val := cfp.max_members;
       FETCH channel_family_perm_cursor INTO cfp;
   END LOOP;
   IF channel_family_perm_cursor%ISOPEN
   THEN
       CLOSE channel_family_perm_cursor;
   END IF;
   IF found IS NULL
   THEN
       RETURN 0;
   END IF;
   IF max_members_val IS NULL
   THEN
       RETURN NULL;
   END IF;
   RETURN max_members_val - current_members_val;
    END available_family_subscriptions;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    IS
    BEGIN
      update rhnPrivateChannelFamily
      set current_members = (
            select   count(sc.server_id)
            from  rhnServerChannel sc,
                  rhnChannelFamilyMembers cfm,
                  rhnServer s
            where s.org_id = org_id_in
               and s.id = sc.server_id
               and cfm.channel_family_id = channel_family_id_in
               and cfm.channel_id = sc.channel_id
            )
         where org_id = org_id_in
            and channel_family_id = channel_family_id_in;
    END update_family_counts;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
   channel_family_id_val NUMBER;
    BEGIN
      SELECT channel_family_id INTO channel_family_id_val
     FROM rhnChannelFamilyMembers
    WHERE channel_id = channel_id_in;
    RETURN rhn_channel.available_family_subscriptions(channel_family_id_val, org_id_in);
    END available_chan_subscriptions;
    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
      cursor permissions is
         select   1
         from  rhnPrivateChannelFamily pcf
         where pcf.org_id = customer_id_in
            and   pcf.channel_family_id = channel_family_id_in;
    begin
      for perm in permissions loop
         set_family_maxmembers(
            customer_id_in,
            channel_family_id_in,
            quantity_in
         );
         rhn_channel.update_family_counts(
            channel_family_id_in,
            customer_id_in
         );
         return;
      end loop;
      insert into rhnPrivateChannelFamily pcf (
            channel_family_id, org_id, max_members, current_members
         ) values (
            channel_family_id_in, customer_id_in, quantity_in, 0
         );
    end;
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
   cursor servers is
       select  server_id from (
      select   rownum row_number, server_id, modified from (
          select  rcfs.server_id,
             rcfs.modified
          from    rhnChannelFamilyServers rcfs
          where   rcfs.customer_id = customer_id_in
             and rcfs.channel_family_id = channel_family_id_in
          order by modified
      )
      where rownum > quantity_in
       );
    begin
   for server in servers
   loop
       rhn_channel.unsubscribe_server_from_family(server.server_id, channel_family_id_in);
   end loop;
   update   rhnPrivateChannelFamily pcf
   set   pcf.max_members = quantity_in
   where pcf.org_id = customer_id_in
      and pcf.channel_family_id = channel_family_id_in;
    end;
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number)
    is
    begin
   delete
   from  rhnServerChannel rsc
   where rsc.server_id = server_id_in
      and channel_id in (
         select   rcfm.channel_id
         from  rhnChannelFamilyMembers rcfm
         where rcfm.channel_family_id = channel_family_id_in);
    end;
    function get_org_id(channel_id_in in number)
    return number
    is
      org_id_out number;
    begin
      select org_id into org_id_out
     from rhnChannel
    where id = channel_id_in;
   return org_id_out;
    end get_org_id;
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number)
    return number
    is
      cursor   families is
         select   1
         from  rhnOrgChannelFamilyPermissions cfp
         where cfp.org_id = org_id_in;
    begin
      for family in families loop
         return 1;
      end loop;
      return 0;
    end;
    function get_org_access(channel_id_in in number, org_id_in in number)
    return number
    is
      throwaway number;
    begin
   select distinct 1 into throwaway
     from rhnChannelFamilyMembers CFM,
          rhnOrgChannelFamilyPermissions CFP
    where cfp.org_id = org_id_in
      and CFM.channel_family_id = CFP.channel_family_id
      and CFM.channel_id = channel_id_in;
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end;
    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
    return number
    is
      org_id number;
    begin
      org_id := rhn_user.get_org_id(user_id_in);
      if role_in = 'manage' and NVL(rhn_channel.get_org_id(channel_id_in), -1) <> org_id
   then
       reason_out := 'channel_not_owned';
       return 0;
   end if;
      if role_in = 'subscribe' and rhn_channel.get_org_access(channel_id_in, org_id) = 0
   then
       reason_out := 'channel_not_available';
       return 0;
   end if;
      if rhn_user.check_role_implied(user_id_in, 'channel_admin') = 1
   then
       reason_out := 'channel_admin';
       return 1;
   end if;
   if role_in = 'subscribe'
   then
       if rhn_channel.org_channel_setting(channel_id_in, org_id, 'not_globally_subscribable') = 0
       then
         reason_out := 'globally_subscribable';
      return 1;
       end if;
   end if;
   reason_out := 'direct_permission';
   return rhn_channel.direct_user_role_check(channel_id_in, user_id_in, role_in);
    end;
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
   throwaway varchar2(256);
    begin
      return rhn_channel.user_role_check_debug(channel_id_in, user_id_in, role_in, throwaway);
    end;
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    begin
        if user_id_in is null then
            return 1;
        end if;
        return user_role_check(channel_id_in, user_id_in, role_in);
    end loose_user_role_check;
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
      throwaway number;
    begin
   select 1 into throwaway
     from rhnChannelPermissionRole CPR,
          rhnChannelPermission CP
    where CP.user_id = user_id_in
      and CP.channel_id = channel_id_in
      and CPR.label = role_in
      and CP.role_id = CPR.id;
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end;
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    return number
    is
      throwaway number;
    begin
   select 1 into throwaway
     from rhnOrgChannelSettingsType OCST,
          rhnOrgChannelSettings OCS
    where OCS.org_id = org_id_in
      and OCS.channel_id = channel_id_in
      and OCST.label = setting_in
      and OCS.setting_id = OCST.id;
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end;
    FUNCTION channel_priority(channel_id_in IN number)
    RETURN number
    IS
         channel_name varchar2(64);
         priority number;
         end_of_life_val date;
         org_id_val number;
    BEGIN
        select name, end_of_life, org_id
        into channel_name, end_of_life_val, org_id_val
        from rhnChannel
        where id = channel_id_in;
        if end_of_life_val is not null then
          return -400;
        end if;
        if channel_name like 'Red Hat Enterprise Linux%' or channel_name like 'RHEL%' then
          priority := 1000;
          if channel_name not like '%Beta%' then
            priority := priority + 1000;
          end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 500
              when channel_name like '%v. 3%' then 400
              when channel_name like '%v. 2%' then 300
              when channel_name like '%v. 1%' then 200
            end;
          priority := priority +
            case
              when (channel_name like '%AS%' and channel_name not like '%Extras%') then 50
              when (channel_name like '%ES%' and channel_name not like '%Extras%') then 40
              when (channel_name like '%WS%' and channel_name not like '%Extras%') then 30
              when (channel_name like '%Desktop%' and channel_name not like '%Extras%') then 20
              when channel_name like '%Extras%' then 10
            end;
          priority := priority +
            case
              when channel_name like '%)' then 5
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif channel_name like 'Red Hat Desktop%' then
            priority := 900;
            if channel_name not like '%Beta%' then
               priority := priority + 50;
            end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 40
              when channel_name like '%v. 3%' then 30
              when channel_name like '%v. 2%' then 20
              when channel_name like '%v. 1%' then 10
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif org_id_val is not null then
          priority := 600;
        else
          priority := 500;
        end if;
      return -priority;
    end channel_priority;
    procedure delete_server_channels(server_id_in in number)
    is
    begin
   update   rhnPrivateChannelFamily
   set   current_members = current_members -1
   where org_id in (
         select   org_id
         from  rhnServer
         where id = server_id_in
      )
      and channel_family_id in (
         select   rcfm.channel_family_id
         from  rhnChannelFamilyMembers rcfm,
            rhnServerChannel rsc
         where rsc.server_id = server_id_in
            and rsc.channel_id = rcfm.channel_id
      );
    end;
   procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)')
   is
   begin
      delete from rhnChannelNewestPackage where channel_id = channel_id_in;
      insert into rhnChannelNewestPackage
         ( channel_id, name_id, evr_id, package_id, package_arch_id )
         (  select   channel_id,
                  name_id, evr_id,
                  package_id, package_arch_id
            from  rhnChannelNewestPackageView
            where channel_id = channel_id_in
         );
                insert into rhnChannelNewestPackageAudit (channel_id, caller)
                    values (channel_id_in, caller_in);
                update rhnChannel
                    set last_modified = greatest(sysdate, last_modified + 1/86400)
                    where id = channel_id_in;
   end;
   procedure update_channel ( channel_id_in in number, invalidate_ss in number := 0,
                              date_to_use in date := sysdate )
   is
   channel_last_modified date;
   last_modified_value date;
   cursor snapshots is
   select  snapshot_id id
   from    rhnSnapshotChannel
   where   channel_id = channel_id_in;
   begin
      select last_modified
      into channel_last_modified
      from rhnChannel
      where id = channel_id_in;
      last_modified_value := date_to_use;
      if last_modified_value <= channel_last_modified then
          last_modified_value := last_modified_value + 1/86400;
      end if;
      update rhnChannel set last_modified = last_modified_value
      where id = channel_id_in;
      if invalidate_ss = 1 then
        for snapshot in snapshots loop
            update rhnSnapshot
            set invalid = lookup_snapshot_invalid_reason('channel_modified')
            where id = snapshot.id;
        end loop;
      end if;
   end update_channel;
   procedure update_channels_by_package ( package_id_in in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelPackage
   where package_id = package_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 1, date_to_use );
      end loop;
   end update_channels_by_package;
   procedure update_channels_by_errata ( errata_id_in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelErrata
   where errata_id = errata_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 0, date_to_use );
      end loop;
   end update_channels_by_errata;
END rhn_channel;
/
SHOW ERRORS;

CREATE OR REPLACE package body rhn_entitlements
is
   body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
   function entitlement_grants_service (
       entitlement_in in varchar2,
       service_level_in in varchar2
   ) return number   is
   begin
      if service_level_in = 'provisioning' then
         if entitlement_in = 'provisioning_entitled' then
            return 1;
         else
            return 0;
         end if;
      elsif service_level_in = 'management' then
         if entitlement_in = 'provisioning_entitled' then
            return 1;
         elsif entitlement_in = 'enterprise_entitled' then
            return 1;
         else
            return 0;
         end if;
      elsif service_level_in = 'monitoring' then
         if entitlement_in = 'monitoring_entitled' then
            return 1;
         end if;
      elsif service_level_in = 'updates' then
         return 1;
      else
         return 0;
      end if;
   end entitlement_grants_service;
   function lookup_entitlement_group (
      org_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) return number is
      cursor server_groups is
         select   sg.id          server_group_id
         from  rhnServerGroup    sg,
               rhnServerGroupType   sgt
         where sgt.label = type_label_in
            and sgt.id = sg.group_type
            and sg.org_id = org_id_in;
   begin
      for sg in server_groups loop
         return sg.server_group_id;
      end loop;
      return rhn_entitlements.create_entitlement_group(
            org_id_in,
            type_label_in
         );
   end lookup_entitlement_group;
   function create_entitlement_group (
      org_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) return number is
      sg_id_val number;
   begin
      select   rhn_server_group_id_seq.nextval
      into  sg_id_val
      from  dual;
      insert into rhnServerGroup (
            id, name, description, max_members, current_members,
            group_type, org_id
         ) (
            select   sg_id_val, sgt.label, sgt.label,
                  0, 0, sgt.id, org_id_in
            from  rhnServerGroupType sgt
            where sgt.label = type_label_in
         );
      return sg_id_val;
   end create_entitlement_group;
   function can_entitle_server ( server_id_in in number, type_label_in in varchar2 )
   return number is
      cursor addon_servergroups (base_label_in in varchar2, addon_label_in in varchar2) is
         select
            addon_id
         from
            rhnSGTypeBaseAddonCompat
         where base_id = lookup_sg_type (base_label_in)
           and addon_id = lookup_sg_type (addon_label_in);
      previous_ent        rhn_entitlements.ents_array;
      is_base_in          char   := 'N';
      is_base_current     char   := 'N';
      i                   number := 0;
      sgid                number := 0;
   begin
      previous_ent := rhn_entitlements.ents_array();
      previous_ent := rhn_entitlements.get_server_entitlement(server_id_in);
      select distinct is_base
      into is_base_in
      from rhnServerGroupType
      where label = type_label_in;
      if previous_ent.count = 0 then
         if (is_base_in = 'Y' and rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid)) then
            return 1;
         else
            return 0;
         end if;
      elsif is_base_in = 'Y' then
         return 0;
      else
         is_base_current := 'N';
         i := 0;
         while is_base_current = 'N' and i <= previous_ent.count
         loop
            i := i + 1;
            select is_base
            into is_base_current
            from rhnServerGroupType
            where label = previous_ent(i);
         end loop;
         if is_base_current  = 'N' then
            return 0;
         end if;
         for addon_servergroup in addon_servergroups  (previous_ent(i), type_label_in) loop
            if rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid) then
               return 1;
            else
               return 0;
            end if;
         end loop;
      end if;
      return 0;
   end can_entitle_server;
   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   ) return number is
      type_label_in_is_base   char(1);
      sgid                    number;
   begin
      begin
         select is_base into type_label_in_is_base
         from rhnServerGroupType
         where label = type_label_in;
      exception
         when no_data_found then
            rhn_exception.raise_exception ( 'invalid_entitlement' );
      end;
      if type_label_in_is_base = 'N' then
         rhn_exception.raise_exception ( 'invalid_entitlement' );
      elsif rhn_entitlements.find_compatible_sg ( server_id_in, type_label_in, sgid ) then
         return 1;
      else
         return 0;
      end if;
   end can_switch_base;
   function find_compatible_sg (
      server_id_in    in   number,
      type_label_in   in   varchar2,
      sgid_out        out  number
   ) return boolean is
      cursor servergroups is
         select sg.id            id
           from rhnServerGroupType             sgt,
                rhnServerGroup                 sg,
                rhnServer                     s,
                rhnServerServerGroupArchCompat ssgac
          where s.id = server_id_in
            and s.org_id = sg.org_id
            and sgt.label = type_label_in
            and sg.group_type = sgt.id
            and ssgac.server_group_type = sgt.id
            and ssgac.server_arch_id = s.server_arch_id
            and not exists (
                     select 1
                      from rhnServerGroupMembers sgm
                     where sgm.server_group_id = sg.id and sgm.server_id = s.id);
   begin
      for servergroup in servergroups loop
         sgid_out := servergroup.id;
         return true;
      end loop;
      sgid_out := 0;
      return false;
   end find_compatible_sg;
   procedure entitle_server (
      server_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) is
      sgid  number := 0;
   begin
      if rhn_entitlements.can_entitle_server (server_id_in, type_label_in) = 1 then
         if rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid) then
            insert into rhnServerHistory ( id, server_id, summary, details )
            values ( rhn_event_id_seq.nextval, server_id_in,
                     'added system entitlement ',
                      case type_label_in
                       when 'enterprise_entitled' then 'Management'
                       when 'sw_mgr_entitled' then 'Update'
                       when 'provisioning_entitled' then 'Provisioning'
                       when 'monitoring_entitled' then 'Monitoring'  end  );
            rhn_server.insert_into_servergroup (server_id_in, sgid);
         else
            rhn_exception.raise_exception ('no_available_server_group');
         end if;
      else
         rhn_exception.raise_exception ('invalid_entitlement');
      end if;
   end entitle_server;
   procedure remove_server_entitlement (
      server_id_in in number,
      type_label_in in varchar2 := 'sw_mgr_entitled'
   ) is
      group_id number;
      type_is_base char;
   begin
      begin
      select   sg.id, sgt.is_base
      into  group_id, type_is_base
      from  rhnServerGroupType sgt,
            rhnServerGroup sg,
            rhnServerGroupMembers sgm,
            rhnServer s
      where s.id = server_id_in
         and s.id = sgm.server_id
         and sgm.server_group_id = sg.id
         and sg.org_id = s.org_id
         and sgt.label = type_label_in
         and sgt.id = sg.group_type;
      if ( type_is_base = 'Y' ) then
         unentitle_server ( server_id_in );
      else
         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case type_label_in
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'  end  );
         rhn_server.delete_from_servergroup(server_id_in, group_id);
         if type_label_in = 'monitoring_entitled' then
           DELETE
             FROM state_change
            WHERE o_id IN (SELECT probe_id
                             FROM rhn_check_probe
                            WHERE host_id = server_id_in);
           DELETE
             FROM time_series
            WHERE SUBSTR(o_id, INSTR(o_id, '-') + 1,
                               (INSTR(o_id, '-', INSTR(o_id, '-') + 1) - INSTR(o_id, '-')) - 1)
              IN (SELECT probe_id
                    FROM rhn_check_probe
                   WHERE host_id = server_id_in);
           DELETE
             FROM rhn_probe
            WHERE recid IN (SELECT probe_id
                              FROM rhn_check_probe
                             WHERE host_id = server_id_in);
         end if;
      end if;
      exception
      when no_data_found then
            rhn_exception.raise_exception('invalid_server_group_member');
      end;
   end remove_server_entitlement;
   procedure unentitle_server (server_id_in in number) is
      cursor servergroups is
         select distinct sgt.label, sg.id server_group_id
         from  rhnServerGroupType sgt,
               rhnServerGroup sg,
               rhnServer s,
               rhnServerGroupMembers sgm
         where s.id = server_id_in
            and s.org_id = sg.org_id
            and sg.group_type = sgt.id
            and sgm.server_group_id = sg.id
            and sgm.server_id = s.id;
   begin
      for servergroup in servergroups loop
         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case servergroup.label
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'  end  );
         rhn_server.delete_from_servergroup ( server_id_in, servergroup.server_group_id );
      end loop;
   end unentitle_server;
   function get_server_entitlement (
      server_id_in in number
   ) return ents_array is
      cursor server_groups is
         select   sgt.label
         from  rhnServerGroupType      sgt,
               rhnServerGroup       sg,
               rhnServerGroupMembers   sgm
         where 1=1
            and sgm.server_id = server_id_in
            and sg.id = sgm.server_group_id
            and sgt.id = sg.group_type
            and sgt.label in (
               'sw_mgr_entitled','enterprise_entitled',
               'provisioning_entitled', 'nonlinux_entitled',
               'monitoring_entitled'
               );
         ent_array ents_array;
   begin
      ent_array := ents_array();
      for sg in server_groups loop
         ent_array.extend;
         ent_array(ent_array.count) := sg.label;
      end loop;
      return ent_array;
   end get_server_entitlement;
   procedure modify_org_service (
      org_id_in in number,
      service_label_in in varchar2,
      enable_in in char
   ) is
      type roles_v is varray(10) of rhnUserGroupType.label%TYPE;
      roles_to_process roles_v;
      cursor roles(role_label_in in varchar2) is
         select   label, id
         from  rhnUserGroupType
         where label = role_label_in;
      cursor org_roles(role_label_in in varchar2) is
         select   1
         from  rhnUserGroup ug,
               rhnUserGroupType ugt
         where ugt.label = role_label_in
            and ug.org_id = org_id_in
            and ugt.id = ug.group_type;
      type ents_v is varray(10) of rhnOrgEntitlementType.label%TYPE;
      ents_to_process ents_v;
      cursor ents(ent_label_in in varchar2) is
         select   label, id
         from  rhnOrgEntitlementType
         where label = ent_label_in;
      cursor org_ents(ent_label_in in varchar2) is
         select   1
         from  rhnOrgEntitlements oe,
               rhnOrgEntitlementType oet
         where oet.label = ent_label_in
            and oe.org_id = org_id_in
            and oet.id = oe.entitlement_id;
      create_row char(1);
   begin
      ents_to_process := ents_v();
      roles_to_process := roles_v();
      if service_label_in = 'enterprise' or service_label_in = 'management' then
         ents_to_process.extend;
         ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'org_admin';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'system_group_admin';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'activation_key_admin';
         if is_satellite() = 1 then
            roles_to_process.extend;
            roles_to_process(roles_to_process.count) := 'org_applicant';
         end if;
      elsif service_label_in = 'provisioning' then
         ents_to_process.extend;
         ents_to_process(ents_to_process.count) := 'rhn_provisioning';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'system_group_admin';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'activation_key_admin';
         if is_satellite() = 1 then
            roles_to_process.extend;
            roles_to_process(roles_to_process.count) := 'config_admin';
         end if;
         if enable_in = 'Y' then
            ents_to_process.extend;
            ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
         end if;
      elsif service_label_in = 'monitoring' then
         ents_to_process.extend;
         ents_to_process(ents_to_process.count) := 'rhn_monitor';
         roles_to_process.extend;
         roles_to_process(roles_to_process.count) := 'monitoring_admin';
      elsif service_label_in = 'nonlinux' then
         ents_to_process.extend;
         ents_to_process(ents_to_process.count) := 'rhn_nonlinux';
         if is_satellite() = 1 then
            roles_to_process.extend;
            roles_to_process(roles_to_process.count) := 'config_admin';
         end if;
      end if;
      if enable_in = 'Y' then
         for i in 1..ents_to_process.count loop
            for ent in ents(ents_to_process(i)) loop
               create_row := 'Y';
               for oe in org_ents(ent.label) loop
                  create_row := 'N';
               end loop;
               if create_row = 'Y' then
                  insert into rhnOrgEntitlements(org_id, entitlement_id)
                     values (org_id_in, ent.id);
               end if;
            end loop;
         end loop;
         for i in 1..roles_to_process.count loop
            for role in roles(roles_to_process(i)) loop
               create_row := 'Y';
               for o_r in org_roles(role.label) loop
                  create_row := 'N';
               end loop;
               if create_row = 'Y' then
                  insert into rhnUserGroup(
                        id, name, description, current_members,
                        group_type, org_id
                     ) (
                        select   rhn_user_group_id_seq.nextval,
                              ugt.name || 's',
                              ugt.name || 's for Org ' ||
                                 o.name || ' ('|| o.id ||')',
                              0, ugt.id, o.id
                        from  rhnUserGroupType ugt,
                              web_customer o
                        where o.id = org_id_in
                           and ugt.id = role.id
                     );
               end if;
            end loop;
         end loop;
      else
         for i in 1..ents_to_process.count loop
            for ent in ents(ents_to_process(i)) loop
               delete from rhnOrgEntitlements
                where org_id = org_id_in
                  and entitlement_id = ent.id;
            end loop;
         end loop;
      end if;
   end modify_org_service;
   procedure set_customer_enterprise (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'enterprise', 'Y');
   end set_customer_enterprise;
   procedure set_customer_provisioning (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'provisioning', 'Y');
   end set_customer_provisioning;
   procedure set_customer_nonlinux (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'nonlinux', 'Y');
   end set_customer_nonlinux;
   procedure unset_customer_enterprise (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'enterprise', 'N');
   end unset_customer_enterprise;
   procedure unset_customer_provisioning (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'provisioning', 'N');
   end unset_customer_provisioning;
   procedure unset_customer_nonlinux (
      customer_id_in in number
   ) is
   begin
      modify_org_service(customer_id_in, 'nonlinux', 'N');
   end unset_customer_nonlinux;
   procedure prune_group (
      group_id_in in number,
      type_in in char,
      quantity_in in number
   ) is
      cursor usergroups is
         select   user_id, user_group_id, ugt.label
         from  rhnUserGroupType  ugt,
               rhnUserGroup      ug,
               rhnUserGroupMembers  ugm
         where 1=1
            and ugm.user_group_id = group_id_in
            and ugm.user_id in (
               select   user_id
               from  (
                  select   rownum row_number,
                        user_id,
                        time
                  from  (
                     select   user_id,
                           modified time
                     from  rhnUserGroupMembers
                     where user_group_id = group_id_in
                     order by time asc
                  )
               )
               where row_number > quantity_in
            )
            and ugm.user_group_id = ug.id
            and ug.group_type = ugt.id;
      cursor servergroups is
         select   server_id, server_group_id, sgt.id group_type_id, sgt.label
         from  rhnServerGroupType      sgt,
               rhnServerGroup       sg,
               rhnServerGroupMembers   sgm
         where 1=1
            and sgm.server_group_id = group_id_in
            and sgm.server_id in (
               select   server_id
               from  (
                  select   rownum row_number,
                        server_id,
                        time
                  from  (
                     select   server_id,
                           modified time
                     from  rhnServerGroupMembers
                     where server_group_id = group_id_in
                     order by time asc
                  )
               )
               where row_number > quantity_in
            )
            and sgm.server_group_id = sg.id
            and sg.group_type = sgt.id;
      type_is_base char;
   begin
      if type_in = 'U' then
         update      rhnUserGroup
            set      max_members = quantity_in
            where id = group_id_in;
         for ug in usergroups loop
            rhn_user.remove_from_usergroup(ug.user_id, ug.user_group_id);
         end loop;
      elsif type_in = 'S' then
         update      rhnServerGroup
            set      max_members = quantity_in
            where id = group_id_in;
         for sg in servergroups loop
            remove_server_entitlement(sg.server_id, sg.label);
            select is_base
            into type_is_base
            from rhnServerGroupType sgt
            where sgt.id = sg.group_type_id;
            if ( type_is_base = 'Y' ) then
               rhn_channel.clear_subscriptions(sg.server_id);
            end if;
         end loop;
      end if;
   end prune_group;
   procedure set_group_count (
      customer_id_in in number,
      type_in in char,
      group_type_in in number,
      quantity_in in number
   ) is
      group_id number;
      quantity number;
   begin
      quantity := quantity_in;
      if quantity is not null and quantity < 0 then
         quantity := 0;
      end if;
      if type_in = 'U' then
         select   rug.id
         into  group_id
         from  rhnUserGroup rug
         where 1=1
            and rug.org_id = customer_id_in
            and rug.group_type = group_type_in;
      elsif type_in = 'S' then
         select   rsg.id
         into  group_id
         from  rhnServerGroup rsg
         where 1=1
            and rsg.org_id = customer_id_in
            and rsg.group_type = group_type_in;
      end if;
      rhn_entitlements.prune_group(
         group_id,
         type_in,
         quantity
      );
   exception
      when no_data_found then
         if type_in = 'U' then
            insert into rhnUserGroup (
                  id, name, description, max_members, current_members,
                  group_type, org_id, created, modified
               ) (
                  select   rhn_user_group_id_seq.nextval, name, name,
                        quantity, 0, id, customer_id_in,
                        sysdate, sysdate
                  from  rhnUserGroupType
                  where id = group_type_in
            );
         elsif type_in = 'S' then
            insert into rhnServerGroup (
                  id, name, description, max_members, current_members,
                  group_type, org_id, created, modified
               ) (
                  select   rhn_server_group_id_seq.nextval, name, name,
                        quantity, 0, id, customer_id_in,
                        sysdate, sysdate
                  from  rhnServerGroupType
                  where id = group_type_in
            );
         end if;
   end set_group_count;
   procedure prune_family (
      customer_id_in in number,
      channel_family_id_in in number,
      quantity_in in number
   ) is
      cursor serverchannels is
         select   sc.server_id,
               sc.channel_id
         from  rhnServerChannel sc,
               rhnChannelFamilyMembers cfm
         where 1=1
            and cfm.channel_family_id = channel_family_id_in
            and cfm.channel_id = sc.channel_id
            and server_id in (
               select   server_id
               from  (
                  select   server_id,
                        time,
                        rownum row_number
                  from  (
                     select   rs.id             server_id,
                           rcfm.modified        time
                     from  rhnServer            rs,
                           rhnServerChannel     rsc,
                           rhnChannelFamilyMembers rcfm
                     where 1=1
                        and rs.org_id = customer_id_in
                        and rs.id = rsc.server_id
                        and rsc.channel_id = rcfm.channel_id
                        and rcfm.channel_family_id =
                           channel_family_id_in
                     order by time asc
                  )
               )
               where row_number > quantity_in
            );
   begin
      if customer_id_in is null then
         return;
      end if;
      update      rhnPrivateChannelFamily
         set      max_members = quantity_in
         where 1=1
            and org_id = customer_id_in
            and channel_family_id = channel_family_id_in;
      for sc in serverchannels loop
         rhn_channel.unsubscribe_server(sc.server_id, sc.channel_id, 1, 1);
      end loop;
   end prune_family;
   procedure set_family_count (
      customer_id_in in number,
      channel_family_id_in in number,
      quantity_in in number
   ) is
      cursor privperms is
         select   1
         from  rhnPrivateChannelFamily
         where org_id = customer_id_in
            and channel_family_id = channel_family_id_in;
      cursor pubperms is
         select   o.id org_id
         from  web_customer o,
               rhnPublicChannelFamily pcf
         where pcf.channel_family_id = channel_family_id_in;
      quantity number;
      done number := 0;
   begin
      quantity := quantity_in;
      if quantity is not null and quantity < 0 then
         quantity := 0;
      end if;
      if customer_id_in is not null then
         for perm in privperms loop
            rhn_entitlements.prune_family(
               customer_id_in,
               channel_family_id_in,
               quantity
            );
            update rhnPrivateChannelFamily
               set max_members = quantity
               where org_id = customer_id_in
                  and channel_family_id = channel_family_id_in;
            return;
         end loop;
         insert into rhnPrivateChannelFamily (
               channel_family_id, org_id, max_members, current_members
            ) values (
               channel_family_id_in, customer_id_in, quantity, 0
            );
         return;
      end if;
      for perm in pubperms loop
         if quantity = 0 then
            rhn_entitlements.prune_family(
               perm.org_id,
               channel_family_id_in,
               quantity
            );
            if done = 0 then
               delete from rhnPublicChannelFamily
                  where channel_family_id = channel_family_id_in;
            end if;
         end if;
         done := 1;
      end loop;
      if done != 1 then
         insert into rhnPublicChannelFamily (
               channel_family_id
            ) values (
               channel_family_id_in
            );
      end if;
   end set_family_count;
   procedure entitle_last_modified_servers (
      customer_id_in in number,
      type_label_in in varchar2,
      quantity_in in number
   ) is
      cursor servers(cid_in in number, quant_in in number) is
         select   server_id
         from  (
                  select   rownum row_number,
                        server_id
                  from  (
                           select   rs.id server_iD
                           from  rhnServer rs
                           where 1=1
                              and rs.org_id = cid_in
                              and not exists (
                                 select   1
                                 from  rhnServerGroup sg,
                                       rhnServerGroupMembers rsgm
                                 where rsgm.server_id = rs.id
                                    and rsgm.server_group_id = sg.id
                                    and sg.group_type is not null
                              )
                           order by modified desc
                        )
               )
         where row_number <= quant_in;
   begin
      for server in servers(customer_id_in, quantity_in) loop
         rhn_entitlements.entitle_server(server.server_id, type_label_in);
      end loop;
   end entitle_last_modified_servers;
   procedure prune_everything (
      customer_id_in in number
   ) is
      cursor everything is
         select   sg.id             id,
               'S'                  type,
               sg.max_members       quantity
         from  rhnServerGroup       sg
         where sg.org_id = customer_id_in
         union
         select   ug.id             id,
               'U'                  type,
               ug.max_members          quantity
         from  rhnUserGroup         ug
         where ug.org_id = customer_id_in
         union (
         select   cfp.channel_family_id   id,
               'C'                  type,
               cfp.max_members         quantity
         from  rhnOrgChannelFamilyPermissions cfp
         where cfp.org_id = customer_id_in
         union
         select   cfm.channel_family_id   id,
               'C'                  type,
               0                 quantity
         from  rhnChannelFamily     cf,
               rhnChannelFamilyMembers cfm,
               rhnServerChannel     sc,
               rhnServer            s
         where s.org_id = customer_id_in
            and s.id = sc.server_id
            and sc.channel_id = cfm.channel_id
            and cfm.channel_family_id = cf.id
            and cf.org_id is not null
            and cf.org_id != customer_id_in
            and not exists (
               select   1
               from  rhnOrgChannelFamilyPermissions cfp
               where cfp.org_id = customer_id_in
                  and cfp.channel_family_id = cfm.channel_family_id
               )
         );
   begin
      for one in everything loop
         if one.type in ('U','S') then
            prune_group(one.id, one.type, one.quantity);
         else
            prune_family(customer_id_in, one.id, one.quantity);
         end if;
      end loop;
   end prune_everything;
   procedure subscribe_newest_servers (
      customer_id_in in number
   ) is
      cursor servers(cid_in in number) is
         select   s.id
         from  rhnServer         s
         where 1=1
            and s.org_id = cid_in
            and not exists (
                  select 1
                  from  rhnChannel        c,
                        rhnServerChannel  sc
                  where sc.server_id = s.id
                     and sc.channel_id = c.id
                     and c.parent_channel is null
               )
         order by s.modified desc;
      channel_id number;
   begin
      for server in servers(customer_id_in) loop
         channel_id := rhn_channel.guess_server_base(server.id);
         if channel_id is not null then
            begin
               rhn_channel.subscribe_server(server.id, channel_id);
               commit;
            exception
               when others then
                  null;
            end;
         end if;
      end loop;
   end subscribe_newest_servers;
end rhn_entitlements;
/
SHOW ERRORS;

CREATE OR REPLACE
PACKAGE BODY rhn_org
IS
	body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';

    FUNCTION find_server_group_by_type(org_id_in NUMBER, group_label_in VARCHAR2)
    RETURN NUMBER
    IS
	server_group       server_group_by_label%ROWTYPE;
    BEGIN
	OPEN server_group_by_label(org_id_in, group_label_in);
	FETCH server_group_by_label INTO server_group;
	CLOSE server_group_by_label;

	RETURN server_group.id;
    END find_server_group_by_type;

	procedure delete_user(user_id_in in number) is
		cursor is_admin is
			select	1
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	ugm.user_id = user_id_in
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id
				and ugt.label = 'org_admin';
		cursor servergroups_needing_admins is
			select	usgp.server_group_id	server_group_id
			from	rhnUserServerGroupPerms	usgp
			where	1=1
				and usgp.user_id = user_id_in
				and not exists (
					select	1
					from	rhnUserServerGroupPerms	sq_usgp
					where	1=1
						and sq_usgp.server_group_id = usgp.server_group_id
						and	sq_usgp.user_id != user_id_in
				);
		cursor servergroups_for_deletion(org_id_in in number) is
			select	sgm.server_id, sgm.server_group_id, sgt.label
			from	rhnServerGroupMembers sgm,
               rhnServerGroupType sgt,
					rhnServerGroup sg
			where	sg.org_id = org_id_in
				and sg.id = sgm.server_group_id
            and sg.group_type = sgt.id;
		cursor servers_for_deletion(org_id_in in number) is
			select	id
			from	rhnServer
			where	org_id = org_id_in;
		cursor snapshots(sgid_in in number) is
			select	snapshot_id id
			from	rhnSnapshotServerGroup
			where	server_group_id = sgid_in;
		cursor messages is
			select	message_id id
			from	rhnUserMessage
			where	user_id = user_id_in;
		users			number;
		our_org_id		number;
		del_check_ok	number;
		other_users		number;
		other_org_admin	number;
      other_user_id  number;
	begin
		select	wc.org_id, xxrh_contact_del_check_num(user_id_in)
		into	our_org_id, del_check_ok
		from	web_contact wc
		where	id = user_id_in;

		-- bail early if we know it won't work
		if del_check_ok != 1 then
			rhn_exception.raise_exception('cannot_delete_user');
			return;
		end if;

		-- find any other users
		begin
			select	id, 1
			into	other_user_id, other_users
			from	web_contact
			where	1=1
				and org_id = our_org_id
				and id != user_id_in
				and rownum = 1;
		exception
			when no_data_found then
				other_users := 0;
		end;

		-- now do org admin stuff
		if other_users != 0 then
			for ignore in is_admin loop
				begin
					select	new_ugm.user_id
					into	other_org_admin
					from	rhnUserGroupMembers	new_ugm,
							rhnUserGroupType	ugt,
							rhnUserGroup		ug,
							rhnUserGroupMembers	ugm
					where	ugm.user_id = user_id_in
						and ugm.user_group_id = ug.id
						and ug.group_type = ugt.id
						and ugt.label = 'org_admin'
						and ug.id = new_ugm.user_group_id
						and new_ugm.user_id != user_id_in
						and rownum = 1;
				exception
					when no_data_found then
						rhn_exception.raise_exception('cannot_delete_user');
				end;

				for sg in servergroups_needing_admins loop
					rhn_user.add_servergroup_perm(other_org_admin,
						sg.server_group_id);
				end loop;
			end loop;
		end if;

		-- and now things for every user
		for message in messages loop
			delete
				from	rhnUserMessage
				where	user_id = user_id_in
					and message_id = message.id;
			begin
				select	1
				into	users
				from	rhnUserMessage
				where	message_id = message.id
					and rownum = 1;
				delete
					from	rhnMessage
					where	id = message.id;
			exception
				when no_data_found then
					null;
			end;
		end loop;
		delete from rhn_command_queue_sessions where contact_id = user_id_in;
		delete from rhn_contact_methods where contact_id = user_id_in;
		delete from rhn_redirects where contact_id = user_id_in;
        delete from rhnUserServerPerms where user_id = user_id_in;
		if other_users != 0 then
			update		rhnRegToken
				set		user_id = nvl(other_org_admin, other_user_id)
				where	org_id = our_org_id
					and user_id = user_id_in;
			begin
				delete from web_contact where id = user_id_in;
			exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		else
			-- so that the delete trigger doesn't bite us
			for sgm in servergroups_for_deletion(our_org_id) loop
				for snapshot in snapshots(sgm.server_group_id) loop
					update rhnSnapshot
						set invalid =
							lookup_snapshot_invalid_reason('sg_removed')
						where id = snapshot.id;
					delete from rhnSnapshotServerGroup
						where snapshot_id = snapshot.id
						and server_group_id = sgm.server_group_id;
				end loop;


            insert into rhnServerHistory ( id, server_id, summary, details )
            values ( rhn_event_id_seq.nextval, sgm.server_id,
                   'removed system entitlement ',
                    case sgm.label
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'  end  );

				rhn_server.delete_from_servergroup(
						sgm.server_id, sgm.server_group_id);
			end loop;
			update rhnServerGroup set group_type = null
				where org_id = our_org_id;
			delete from rhnServerGroup where org_id = our_org_id;
			for s in servers_for_deletion(our_org_id) loop
				delete_server(s.id);
			end loop;
			begin
				delete from rhn_check_suites where customer_id = our_org_id;
				delete from rhn_command_target where customer_id = our_org_id;
				delete from rhn_contact_groups where customer_id = our_org_id;
				delete from rhn_notification_formats
					where customer_id = our_org_id;
				delete from rhn_probe where customer_id = our_org_id;
				delete from rhn_redirects where customer_id = our_org_id;
				delete from rhn_sat_cluster where customer_id = our_org_id;
				delete from rhn_schedules where customer_id = our_org_id;
				delete from web_contact where id = user_id_in;
				delete from web_customer where id = our_org_id;
			exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		end if;
		return;
	end delete_user;
END rhn_org;
/
SHOW ERRORS;

CREATE OR REPLACE package body rhn_server
is
    function system_service_level(
      server_id_in in number,
   service_level_in in varchar2
    ) return number is
    cursor ents is
      select label from rhnServerEntitlementView
      where server_id = server_id_in;
    retval number := 0;
    begin
         for ent in ents loop
            retval := rhn_entitlements.entitlement_grants_service (ent.label, service_level_in);
            if retval = 1 then
               return retval;
            end if;
         end loop;
         return retval;
    end system_service_level;
    function can_change_base_channel(server_id_in IN NUMBER)
    return number
    is
      throwaway number;
    begin
   select 1 into throwaway
     from rhnServer S
    where S.id = server_id_in
      and not exists (select 1 from rhnSatelliteInfo SI where SI.server_id = S.id)
      and not exists (select 1 from rhnProxyInfo PI where PI.server_id = S.id);
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end can_change_base_channel;
    procedure set_custom_value(
      server_id_in in number,
   user_id_in in number,
   key_label_in in varchar2,
   value_in in varchar2
    ) is
      key_id_val number;
    begin
      select CDK.id into key_id_val
     from rhnCustomDataKey CDK,
          rhnServer S
    where S.id = server_id_in
      and S.org_id = CDK.org_id
      and CDK.label = key_label_in;
   begin
       insert into rhnServerCustomDataValue (server_id, key_id, value, created_by, last_modified_by)
       values (server_id_in, key_id_val, value_in, user_id_in, user_id_in);
   exception
       when DUP_VAL_ON_INDEX
         then
      update rhnServerCustomDataValue
         set value = value_in,
             last_modified_by = user_id_in
       where server_id = server_id_in
         and key_id = key_id_val;
   end;
    end set_custom_value;
    procedure bulk_set_custom_value(
      key_label_in in varchar2,
   value_in in varchar2,
   set_label_in in varchar2,
   set_uid_in in number
    ) is
    begin
      for server in rhn_set.set_iterator(set_label_in, set_uid_in)
   loop
       if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
         rhn_server.set_custom_value(server.element, set_uid_in, key_label_in, value_in);
       end if;
   end loop server;
    end bulk_set_custom_value;
    procedure bulk_snapshot_tag(
      org_id_in in number,
        tagname_in in varchar2,
   set_label_in in varchar2,
   set_uid_in in number
    ) is
      snapshot_id number;
    begin
      for server in rhn_set.set_iterator(set_label_in, set_uid_in)
   loop
       if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
         begin
             select max(id) into snapshot_id
             from rhnSnapshot
             where server_id = server.element;
         exception
             when NO_DATA_FOUND then
            rhn_server.snapshot_server(server.element, 'tagging system:  ' || tagname_in);
         select max(id) into snapshot_id
         from rhnSnapshot
         where server_id = server.element;
      end;
      begin
          rhn_server.tag_snapshot(snapshot_id, org_id_in, tagname_in);
      exception
          when DUP_VAL_ON_INDEX
            then
         null;
      end;
       end if;
   end loop server;
    end bulk_snapshot_tag;
    procedure tag_delete(
      server_id_in in number,
   tag_id_in in number
    ) is
      cursor snapshots is
      select   snapshot_id
      from  rhnSnapshotTag
      where tag_id = tag_id_in;
   tag_id_tmp number;
    begin
      select   id into tag_id_tmp
   from  rhnTag
   where id = tag_id_in
   for update;
   delete
      from  rhnSnapshotTag
      where server_id = server_id_in
         and tag_id = tag_id_in;
   for snapshot in snapshots loop
      return;
   end loop;
   delete
      from rhnTag
      where id = tag_id_in;
    end tag_delete;
    procedure tag_snapshot(
        snapshot_id_in in number,
   org_id_in in number,
   tagname_in in varchar2
    ) is
    begin
      insert into rhnSnapshotTag (snapshot_id, server_id, tag_id)
   select snapshot_id_in, server_id, lookup_tag(org_id_in, tagname_in)
   from rhnSnapshot
   where id = snapshot_id_in;
    end tag_snapshot;
    procedure bulk_snapshot(
      reason_in in varchar2,
   set_label_in in varchar2,
   set_uid_in in number
    ) is
    begin
      for server in rhn_set.set_iterator(set_label_in, set_uid_in)
   loop
          if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
         rhn_server.snapshot_server(server.element, reason_in);
       end if;
   end loop server;
    end bulk_snapshot;
    procedure snapshot_server(
      server_id_in in number,
   reason_in in varchar2
    ) is
      snapshot_id number;
   cursor revisions is
      select distinct
         cr.id
      from  rhnConfigRevision cr,
         rhnConfigFileName cfn,
         rhnConfigFile     cf,
         rhnConfigChannel  cc,
         rhnServerConfigChannel  scc
      where 1=1
         and scc.server_id = server_id_in
         and scc.config_channel_id = cc.id
         and cc.id = cf.config_channel_id
         and cf.id = cr.config_file_id
         and cr.id = cf.latest_config_revision_id
         and cf.config_file_name_id = cfn.id
         and cf.id = lookup_first_matching_cf(scc.server_id, cfn.path);
    begin
      select rhn_snapshot_id_seq.nextval into snapshot_id from dual;
   insert into rhnSnapshot (id, org_id, server_id, reason) (
      select   snapshot_id,
         s.org_id,
         server_id_in,
         reason_in
      from  rhnServer s
      where s.id = server_id_in
   );
   insert into rhnSnapshotChannel (snapshot_id, channel_id) (
      select   snapshot_id, sc.channel_id
      from  rhnServerChannel sc
      where sc.server_id = server_id_in
   );
   insert into rhnSnapshotServerGroup (snapshot_id, server_group_id) (
      select   snapshot_id, sgm.server_group_id
      from  rhnServerGroupMembers sgm
      where sgm.server_id = server_id_in
   );
   insert into rhnSnapshotPackage (snapshot_id, nevra_id) (
      select   distinct snapshot_id,
         lookup_package_nevra(sp.name_id, sp.evr_id, null)
      from  rhnServerPackage sp
      where sp.server_id = server_id_in
   );
   insert into rhnSnapshotConfigChannel ( snapshot_id, config_channel_id ) (
      select   snapshot_id, scc.config_channel_id
      from  rhnServerConfigChannel scc
      where server_id = server_id_in
   );
   for revision in revisions loop
      insert into rhnSnapshotConfigRevision (
            snapshot_id, config_revision_id
         ) values (
            snapshot_id, revision.id
         );
   end loop;
    end snapshot_server;
    procedure remove_action(
      server_id_in in number,
   action_id_in in number
    ) is
      cursor chained_actions is
      select   id, prerequisite
      from  rhnAction
      start with id = action_id_in
      connect by prior id = prerequisite
      order by prerequisite desc;
   cursor sessions is
      select   s.id
      from  rhnKickstartSession s
      where server_id_in in (s.old_server_id, s.new_server_id)
         and s.action_id = action_id_in
         and not exists (
            select   1
            from  rhnKickstartSessionState ss
            where ss.id = s.state_id
               and ss.label in ('failed','complete')
         );
   type chain_end_type is table of number index by binary_integer;
   chain_ends chain_end_type;
   i number;
   prereq number := 1;
    begin
   select   prerequisite
   into  prereq
   from  rhnAction
   where id = action_id_in;
   if prereq is not null then
      rhn_exception.raise_exception('action_is_child');
   end if;
   i := 0;
   for action in chained_actions loop
      if action.prerequisite is null then
         chain_ends(i) := action.id;
         i := i + 1;
      else
         delete from rhnServerAction
            where server_id = server_id_in
            and action_id = action.id;
      end if;
   end loop;
   i := chain_ends.first;
   while i is not null loop
      delete from rhnServerAction
         where server_id = server_id_in
         and action_id = chain_ends(i);
      i := chain_ends.next(i);
   end loop;
   for s in sessions loop
      update rhnKickstartSession
         set   state_id = (
               select   id
               from  rhnKickstartSessionState
               where label = 'failed'
            ),
            action_id = null
         where id = s.id;
      set_ks_session_history_message(s.id, 'failed', 'Kickstart cancelled due to action removal');
   end loop;
    end remove_action;
    function check_user_access(server_id_in in number, user_id_in in number)
    return number
    is
      has_access number;
    begin
        select 1 into has_access
     from rhnServer S,
          web_contact wc
    where wc.org_id = s.org_id
      and s.id = server_id_in
      and wc.id = user_id_in;
      if rhn_user.check_role(user_id_in, 'org_admin') = 1
   then
       return 1;
   end if;
      select 1 into has_access
     from rhnServerGroupMembers SGM,
          rhnUserServerGroupPerms USG
    where SGM.server_group_id = USG.server_group_id
      and SGM.server_id = server_id_in
      and USG.user_id = user_id_in
      and rownum = 1;
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end check_user_access;
    procedure insert_into_servergroup (
      server_id_in in number,
      server_group_id_in in number
    ) is
      cursor sg_users is
         select   user_id id
         from  rhnUserServerGroupPerms
         where server_group_id = server_group_id_in;
      used_slots number;
      max_slots number;
      org_id number;
      mgmt_available number;
      mgmt_upgrade number;
      mgmt_sgid number;
      prov_available number;
      prov_upgrade number;
      prov_sgid number;
      group_label rhnServerGroupType.label%TYPE;
      group_type number;
   begin
      select   sg.group_type, sg.org_id, sg.current_members, sg.max_members
      into  group_type, org_id, used_slots, max_slots
      from  rhnServerGroup sg
      where sg.id = server_group_id_in
      for update of sg.current_members;
      if group_type is null then
         if used_slots >= max_slots then
            rhn_exception.raise_exception('servergroup_max_members');
         end if;
         insert into rhnServerGroupMembers(
               server_id, server_group_id
            ) values (
               server_id_in, server_group_id_in
            );
         update rhnServerGroup
            set current_members = current_members + 1
            where id = server_group_id_in;
         for u in sg_users loop
            rhn_cache.update_perms_for_user(u.id);
         end loop;
         return;
      end if;
      select   label
      into  group_label
      from  rhnServerGroupType   sgt
      where sgt.id = group_type;
      if group_label in ('sw_mgr_entitled','enterprise_entitled','monitoring_entitled',
         'provisioning_entitled') then
         if used_slots >= max_slots then
            rhn_exception.raise_exception('servergroup_max_members');
         end if;
         insert into rhnServerGroupMembers(
               server_id, server_group_id
            ) values (
               server_id_in, server_group_id_in
            );
         update rhnServerGroup
            set current_members = current_members + 1
            where id = server_group_id_in;
         return;
      end if;
   end;
   function insert_into_servergroup_maybe (
      server_id_in in number,
      server_group_id_in in number
   ) return number is
      retval number := 0;
      cursor servergroups is
         select   s.id  server_id,
               sg.id server_group_id
         from  rhnServerGroup sg,
               rhnServer      s
         where s.id = server_id_in
            and sg.id = server_group_id_in
            and s.org_id = sg.org_id
            and not exists (
               select   1
               from  rhnServerGroupMembers sgm
               where sgm.server_id = s.id
                  and sgm.server_group_id = sg.id
            );
   begin
      for sgm in servergroups loop
         rhn_server.insert_into_servergroup(sgm.server_id, sgm.server_group_id);
         retval := retval + 1;
      end loop;
      return retval;
   end insert_into_servergroup_maybe;
   procedure insert_set_into_servergroup (
      server_group_id_in in number,
      user_id_in in number,
      set_label_in in varchar2
   ) is
      cursor servers is
         select   st.element  id
         from  rhnSet      st
         where st.user_id = user_id_in
            and st.label = set_label_in
            and exists (
               select   1
               from  rhnUserManagedServerGroups umsg
               where umsg.server_group_id = server_group_id_in
                  and umsg.user_id = user_id_in
               )
            and not exists (
               select   1
               from  rhnServerGroupMembers sgm
               where sgm.server_id = st.element
                  and sgm.server_group_id = server_group_id_in
            );
   begin
      for s in servers loop
         rhn_server.insert_into_servergroup(s.id, server_group_id_in);
      end loop;
   end insert_set_into_servergroup;
    procedure delete_from_servergroup (
      server_id_in in number,
      server_group_id_in in number
    ) is
      cursor sg_users is
         select   user_id id
         from  rhnUserServerGroupPerms
         where server_group_id = server_group_id_in;
      oid number;
      mgmt_sgid number;
      label rhnServerGroupType.label%TYPE;
      group_type number;
   begin
      begin
         select   sg.group_type, sg.org_id
         into  group_type, oid
         from  rhnServerGroupMembers   sgm,
               rhnServerGroup       sg
         where sg.id = server_group_id_in
            and sg.id = sgm.server_group_id
            and sgm.server_id = server_id_in
         for update of sg.current_members;
      exception
         when no_data_found then
            rhn_exception.raise_exception('server_not_in_group');
      end;
      if group_type is null then
         delete from rhnServerGroupMembers
            where server_group_id = server_group_id_in
            and   server_id = server_id_in;
         update rhnServerGroup
            set current_members = current_members - 1
            where id = server_group_id_in;
         for u in sg_users loop
            rhn_cache.update_perms_for_user(u.id);
         end loop;
         return;
      end if;
      select   sgt.label
      into  label
      from  rhnServerGroupType sgt
      where sgt.id = group_type;
      if label in ('sw_mgr_entitled','enterprise_entitled', 'provisioning_entitled', 'monitoring_entitled') then
         delete from rhnServerGroupMembers
            where server_group_id = server_group_id_in
            and   server_id = server_id_in;
         update rhnServerGroup
            set current_members = current_members - 1
            where id = server_group_id_in;
      end if;
   end;
   procedure delete_set_from_servergroup (
      server_group_id_in in number,
      user_id_in in number,
      set_label_in in varchar2
   ) is
      cursor servergroups is
         select   sgm.server_id, sgm.server_group_id
         from  rhnSet st,
               rhnServerGroupMembers sgm
         where sgm.server_group_id = server_group_id_in
            and st.user_id = user_id_in
            and st.label = set_label_in
            and sgm.server_id = st.element
            and exists (
               select   1
               from  rhnUserManagedServerGroups usgp
               where usgp.server_group_id = server_group_id_in
                  and usgp.user_id = user_id_in
            );
   begin
      for sgm in servergroups loop
         rhn_server.delete_from_servergroup(sgm.server_id, server_group_id_in);
      end loop;
   end delete_set_from_servergroup;
   procedure clear_servergroup (
      server_group_id_in in number
   ) is
      cursor servers is
         select   sgm.server_id  id
         from  rhnServerGroupMembers sgm
         where sgm.server_group_id = server_group_id_in;
   begin
      for s in servers loop
         rhn_server.delete_from_servergroup(s.id, server_group_id_in);
      end loop;
   end clear_servergroup;
   procedure delete_from_org_servergroups (
      server_id_in in number
   ) is
      cursor servergroups is
         select   sgm.server_group_id id
         from  rhnServerGroup sg,
               rhnServerGroupMembers sgm
         where sgm.server_id = server_id_in
            and sgm.server_group_id = sg.id
            and sg.group_type is null;
   begin
      for sg in servergroups loop
         rhn_server.delete_from_servergroup(server_id_in, sg.id);
      end loop;
   end delete_from_org_servergroups;
   function get_ip_address (
      server_id_in in number
   ) return varchar2 is
      cursor interfaces is
         select   name, ip_addr
         from  rhnServerNetInterface
         where server_id = server_id_in
            and name = 'eth0';
      cursor addresses is
         select   ipaddr ip_addr
         from  rhnServerNetwork
         where server_id = server_id_in
            and ipaddr != '127.0.0.1';
   begin
      for iface in interfaces loop
         return iface.ip_addr;
      end loop;
      for addr in addresses loop
         return addr.ip_addr;
      end loop;
      return NULL;
   end get_ip_address;
end rhn_server;
/
SHOW ERRORS;

CREATE OR REPLACE package body rhn_user
is
   body_version varchar2(100) := '$Id: satellite-3.7-to-4.0.sql 101839 2006-09-06 18:23:03Z jslagle $';
    function check_role(user_id_in in number, role_in in varchar2)
    return number
    is
      throwaway number;
    begin
   select 1 into throwaway
     from rhnUserGroupType UGT,
          rhnUserGroup UG,
          rhnUserGroupMembers UGM
    where UGM.user_id = user_id_in
      and UGM.user_group_id = UG.id
      and UG.group_type = UGT.id
      and UGT.label = role_in;
   return 1;
    exception
      when no_data_found
       then
       return 0;
    end check_role;
    function check_role_implied(user_id_in in number, role_in in varchar2)
    return number
    is
      throwaway number;
    begin
      if rhn_user.check_role(user_id_in, role_in) = 1
   then
       return 1;
      end if;
   if role_in = 'config_admin' and rhn_user.check_role(user_id_in, 'org_admin') = 1
   then
       return 1;
   end if;
   if role_in = 'channel_admin' and rhn_user.check_role(user_id_in, 'org_admin') = 1
   then
       return 1;
   end if;
   return 0;
    end check_role_implied;
    function get_org_id(user_id_in in number)
    return number
    is
      org_id_out number;
    begin
      select org_id into org_id_out
     from web_contact
    where id = user_id_in;
   return org_id_out;
    end get_org_id;
   function find_mailable_address(user_id_in in number)
   return varchar2 is
      PRAGMA AUTONOMOUS_TRANSACTION;
      cursor addrs is
         select   ea.state_id, ea.address
         from  rhnEmailAddressState eas,
               rhnEmailAddress ea
         where ea.user_id = user_id_in
            and eas.label = 'verified'
            and ea.state_id = eas.id
         union all
         select   ea.state_id, ea.address
         from  rhnEmailAddressState eas,
               rhnEmailAddress ea
         where ea.user_id = user_id_in
            and eas.label = 'unverified'
            and ea.state_id = eas.id
         union all
         select   ea.state_id, ea.address
         from  rhnEmailAddressState eas,
               rhnEmailAddress ea
         where ea.user_id = user_id_in
            and eas.label = 'pending'
            and ea.state_id = eas.id
         union all
         select   ea.state_id, ea.address
         from  rhnEmailAddressState eas,
               rhnEmailAddress ea
         where ea.user_id = user_id_in
            and eas.label = 'pending_warned'
            and ea.state_id = eas.id
         union all
         select   ea.state_id, ea.address
         from  rhnEmailAddressState eas,
               rhnEmailAddress ea
         where ea.user_id = user_id_in
            and eas.label = 'needs_verifying'
            and ea.state_id = eas.id
         union all
         select   -1 state_id,
               email address
         from  web_user_personal_info
         where web_user_id = user_id_in;
      retval rhnEmailAddress.address%TYPE;
   begin
      for addr in addrs loop
         retval := addr.address;
         if addr.address is null then
            update web_user_contact_permission
               set email = 'N'
               where web_user_id = user_id_in;
            commit;
            return null;
         end if;
         if addr.state_id = -1 then
            insert into rhnEmailAddress (
                  id, address,
                  user_id, state_id
               ) (
                  select   rhn_eaddress_id_seq.nextval, addr.address,
                        user_id_in, eas.id
                  from  rhnEmailAddressState eas
                  where eas.label = 'unverified'
               );
         end if;
         commit;
         return retval;
      end loop;
      return null;
   end;
   procedure add_servergroup_perm(
      user_id_in in number,
      server_group_id_in in number
   ) is
      cursor   orgs_match is
         select   1
         from  rhnServerGroup sg,
               web_contact u
         where u.id = user_id_in
            and sg.id = server_group_id_in
            and sg.org_id = u.org_id;
   begin
      for okay in orgs_match loop
         insert into rhnUserServerGroupPerms(user_id, server_group_id)
            values (user_id_in, server_group_id_in);
         rhn_cache.update_perms_for_user(user_id_in);
         return;
      end loop;
      rhn_exception.raise_exception('usgp_different_orgs');
   exception when dup_val_on_index then
      rhn_exception.raise_exception('usgp_already_allowed');
   end add_servergroup_perm;
   procedure remove_servergroup_perm(
      user_id_in in number,
      server_group_id_in in number
   ) is
      cursor perms is
         select   1
         from  rhnUserServerGroupPerms
         where user_id = user_id_in
            and server_group_id = server_group_id_in;
   begin
      for perm in perms loop
         delete from rhnUserServerGroupPerms
            where user_id = user_id_in
               and server_group_id = server_group_id_in;
         rhn_cache.update_perms_for_user(user_id_in);
         return;
      end loop;
      rhn_exception.raise_exception('usgp_not_allowed');
   end remove_servergroup_perm;
   procedure add_to_usergroup(
      user_id_in in number,
      user_group_id_in in number
   ) is
      cursor perm_granting_usergroups is
         select   user_group_id_in
         from  rhnUserGroup      ug,
               rhnUserGroupType  ugt
         where ugt.label in ('org_admin') -- and server_group_admin ?
            and ug.id = user_group_id_in
            and ug.group_type = ugt.id;
   begin
      insert into rhnUserGroupMembers(user_id, user_group_id)
         values (user_id_in, user_group_id_in);
      for ug in perm_granting_usergroups loop
         rhn_cache.update_perms_for_user(user_id_in);
         return;
      end loop;
   end add_to_usergroup;
   procedure add_users_to_usergroups(
      user_id_in in number
   ) is
      cursor ugms is
         select   element user_id,
               element_two user_group_id
         from  rhnSet
         where user_id = user_id_in
            and label = 'user_group_list';
   begin
      for ugm in ugms loop
         rhn_user.add_to_usergroup(ugm.user_id, ugm.user_group_id);
      end loop;
   end add_users_to_usergroups;
   procedure remove_from_usergroup(
      user_id_in in number,
      user_group_id_in in number
   ) is
      cursor perm_granting_usergroups is
         select   label
         from  rhnUserGroupType  ugt,
               rhnUserGroupMembers  ugm,
               rhnUserGroup      ug
         where 1=1
            and ug.id = user_group_id_in
            and ugm.user_group_id = user_group_id_in
            and ug.group_type = ugt.id
            and ugm.user_id = user_id_in;
   begin
      for ug in perm_granting_usergroups loop
         delete from rhnUserGroupMembers
            where user_id = user_id_in
               and user_group_id = user_group_id_in;
         if ug.label in ('org_admin') then
            rhn_cache.update_perms_for_user(user_id_in);
         end if;
      end loop;
   end remove_from_usergroup;
   procedure remove_users_from_servergroups(
      user_id_in in number
   ) is
      cursor ugms is
         select   element user_id,
               element_two user_group_id
         from  rhnSet
         where user_id = user_id_in
            and label = 'user_group_list';
   begin
      for ugm in ugms loop
         rhn_user.remove_from_usergroup(ugm.user_id, ugm.user_group_id);
      end loop;
   end remove_users_from_servergroups;
end rhn_user;
/
SHOW ERRORS;

-- Procedures

CREATE OR REPLACE PROCEDURE rhn_synch_probe_state
is
begin
    update
        rhn_probe_state
    set state = 'PENDING',
        output = 'Awaiting update'
    where last_check < (
        select (
            sysdate - greatest(15 / 60 / 24,
            ((3 * rhn_deployed_probe.check_interval_minutes) / 60 / 24)))
        from rhn_deployed_probe
        where rhn_deployed_probe.recid = rhn_probe_state.probe_id
    );
    update rhn_multi_scout_threshold t
    set (scout_warning_threshold, scout_critical_threshold)=(
        select
            decode(scout_warning_threshold_is_all,0,
                scout_warning_threshold,count(scout_id)),
            decode(scout_crit_threshold_is_all,0,
                scout_critical_threshold,count(scout_id))
        from rhn_probe_state p
        where t.probe_id=p.probe_id
          and state in ('OK', 'WARNING', 'CRITICAL')
        group by t.probe_id
    );
end rhn_synch_probe_state;
/
SHOW ERRORS;

CREATE OR REPLACE function
lookup_feature_type(label_in in varchar2)
return number
deterministic
is
   feature_id number;
begin
   select   id
   into  feature_id
   from  rhnFeature
   where label = label_in;
   return feature_id;
exception
        when no_data_found then
            rhn_exception.raise_exception('invalid_feature');
end;
/
SHOW ERRORS;

CREATE OR REPLACE FUNCTION
LOOKUP_PACKAGE_ARCH(label_in IN VARCHAR2)
RETURN NUMBER
DETERMINISTIC
IS
   package_arch_id      NUMBER;
BEGIN
   if label_in is null then
      return null;
   end if;
   SELECT id
          INTO package_arch_id
          FROM rhnPackageArch
         WHERE label = label_in;
   RETURN package_arch_id;
EXCEPTION
        WHEN NO_DATA_FOUND THEN
            rhn_exception.raise_exception('package_arch_not_found');
END;
/
SHOW ERRORS;


create or replace
procedure delete_server (
	server_id_in in number
) is
	cursor servergroups is
		select	server_id, server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			-- these config channel types are reserved
			-- for use by a single server, so we don't
			-- need to check for other servers subscribed
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl
		where	spfl.server_id = server_id_in
			and not exists (
				select	1
				from	rhnServerPreserveFileList
				where	file_list_id = spfl.file_list_id
					and server_id != server_id_in
				union
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			);
	cluster_id number;
begin
	rhn_channel.delete_server_channels(server_id_in);
	-- rhn_channel.clear_subscriptions(server_id_in);

	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;

	for configchannel in configchannels loop
		rhn_config.delete_channel(configchannel.id);
	end loop;
	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;
	-- we're handling this instead of letting an "on delete
	-- set null" do it so that we don't run the risk
	-- of setting off the triggers and killing us with a
	-- mutating table

	-- we cna't use a cursor to do this because we need "case"
	-- in a select to do that, and 8.1.7.3.0 doesn't support it.
	update rhnKickstartSession
		set old_server_id = null
		where old_server_id = server_id_in;
	update rhnKickstartSession
		set new_server_id = null
		where new_server_id = server_id_in;

	rhn_channel.clear_subscriptions(server_id_in,1);
	-- We're deleting everything with a foreign key to rhnServer
	-- here, now.  I'm hoping this will help aleviate our deadlock
	-- problem.

	delete from rhnActionConfigChannel where server_id = server_id_in;
	delete from rhnActionConfigRevision where server_id = server_id_in;
	delete from rhnActionPackageRemovalFailure where server_id = server_id_in;
	delete from rhnChannelFamilyLicenseConsent where server_id = server_id_in;
	delete from rhnClientCapability where server_id = server_id_in;
	delete from rhnCpu where server_id = server_id_in;
	-- there's still a cascade here, because the constraint keeps the
	-- table locked for too long to rebuild it.  Ugh...
	delete from rhnDevice where server_id = server_id_in;
	delete from rhnProxyInfo where server_id = server_id_in;
	delete from rhnRam where server_id = server_id_in;
	delete from rhnRegToken where server_id = server_id_in;
	delete from rhnSNPServerQueue where server_id = server_id_in;
	delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	delete from rhnSatelliteInfo where server_id = server_id_in;
	-- this cascades to rhnActionConfigChannel and rhnActionConfigFileName
	delete from rhnServerAction where server_id = server_id_in;
	delete from rhnServerActionPackageResult where server_id = server_id_in;
	delete from rhnServerActionScriptResult where server_id = server_id_in;
	delete from rhnServerActionVerifyResult where server_id = server_id_in;
	delete from rhnServerActionVerifyMissing where server_id = server_id_in;
	-- counts are handled above.  this should be a delete_ function.
	delete from rhnServerChannel where server_id = server_id_in;
	delete from rhnServerConfigChannel where server_id = server_id_in;
	delete from rhnServerCustomDataValue where server_id = server_id_in;
	delete from rhnServerDMI where server_id = server_id_in;
	delete from rhnServerMessage where server_id = server_id_in;
	-- this gets rhnServerMessage (only) on cascade; it's handled just above
	delete from rhnServerEvent where server_id = server_id_in;
	delete from rhnServerHistory where server_id = server_id_in;
	delete from rhnServerInfo where server_id = server_id_in;
	delete from rhnServerInstallInfo where server_id = server_id_in;
	delete from rhnServerLocation where server_id = server_id_in;
	delete from rhnServerLock where server_id = server_id_in;
	delete from rhnServerNeededPackageCache where server_id = server_id_in;
	delete from rhnServerNeededErrataCache where server_id = server_id_in;
	delete from rhnServerNetwork where server_id = server_id_in;
	delete from rhnServerNotes where server_id = server_id_in;
	-- I'm not removing the foreign key from rhnServerPackage; that'll
	-- take forever.  Do the delete anyway.
	delete from rhnServerPackage where server_id = server_id_in;
	delete from rhnServerTokenRegs where server_id = server_id_in;
	delete from rhnSnapshotTag where server_id = server_id_in;
	-- this cascades to:
	--   rhnSnapshotChannel, rhnSnapshotConfigChannel, rhnSnapshotPackage,
	--   rhnSnapshotConfigRevision, rhnSnapshotServerGroup,
	--   rhnSnapshotTag.
	-- We may want to consider delete_snapshot() at some point, but
	--   I don't think we need to yet.
	delete from rhnSnapshot where server_id = server_id_in;
	delete from rhnTransaction where server_id = server_id_in;
	delete from rhnUserServerPrefs where server_id = server_id_in;
	-- hrm, this one's interesting... we _probably_ should delete
	-- everything for the parent server_id when we delete the proxy,
	-- but we don't currently.
	delete from rhnServerPath where server_id_in in (server_id, proxy_server_id);
	delete from rhnUserServerPerms where server_id = server_id_in;

	delete from rhn_interface_monitoring where server_id = server_id_in;
	delete from rhnServerNetInterface where server_id = server_id_in;
	delete from rhn_server_monitoring_info where recid = server_id_in;

	delete from rhnAppInstallSession where server_id = server_id_in;
	delete from rhnServerUuid where server_id = server_id_in;
    -- first we delete all the probes running directly against this
    -- system
    DELETE FROM rhn_probe_state PS WHERE PS.probe_id in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );

    DELETE FROM rhn_probe P  WHERE P.recid in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );
    -- Now we delete any probes that were using this Server
    -- as a Proxy Scout.
    DELETE FROM rhn_probe_state PS
      WHERE PS.probe_id in
      (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.sat_cluster_id =
    (SELECT SN.sat_cluster_id
       FROM rhn_sat_node SN
      WHERE SN.server_id = server_id_in));

    DELETE FROM rhn_probe P
       WHERE P.recid in
      (SELECT CP.probe_id
         FROM rhn_check_probe CP
         WHERE CP.sat_cluster_id =
      (SELECT SN.sat_cluster_id
         FROM rhn_sat_node SN
         WHERE SN.server_id = server_id_in));

	delete from rhn_check_probe where host_id = server_id_in;
	delete from rhn_host_probe where host_id = server_id_in;

    delete from rhn_sat_cluster where recid in
      ( select sat_cluster_id from rhn_sat_node where server_id = server_id_in );

	delete from rhn_sat_node where server_id = server_id_in;

	-- now get rhnServer itself.
	delete
	from	rhnServer
		where id = server_id_in;

	delete
	from	rhnSet
	where	1=1
		and user_id in (
			select	wc.id
			from	rhnServer rs,
				web_contact wc
			where	rs.id = server_id_in
				and rs.org_id = wc.org_id
		)
		and label = 'system_list'
		and element = server_id_in;
end delete_server;
/
show errors;

create or replace
procedure delete_server_bulk (
	user_id_in in number
) is
	cursor systems is
		select	s.element id
		from	rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list';
	cursor servergroups is
		select	sgm.server_group_id, sgm.server_id
		from	rhnServerGroupMembers sgm,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = sgm.server_id;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc,
			rhnSet s
		where	1=1
			and s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = scc.server_id
			and scc.config_channel_id = cc.id
			-- these config channel types are reserved
			-- for use by a single server, so we don't
			-- need to check for other servers subscribed
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = spfl.server_id
			and not exists (
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			)
			and not exists (
				-- this has the org_id just to make the set -much- smaller.
				select	spfl.server_id
				from	rhnServerPreserveFileList spfl,
					rhnServer s,
					web_contact u
				where	u.id = user_id_in
					and u.org_id = s.org_id
					and s.id = spfl.server_id
					and spfl.file_list_id = spfl.file_list_id
				minus
				select	element server_id
				from	rhnSet
				where	s.user_id = user_id_in
					and s.label = 'system_list'
			);
	cluster_id number;
begin
	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;

	for cc in configchannels loop
		rhn_config.delete_channel(cc.id);
	end loop;

	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;

	-- it's too bad you can't use "current of" with deletes
	for s in systems loop
		-- we can't use a cursor like planned to for delete_server
		-- to fix rhnKickstartSession entries, because
		-- we miss entries where old_server_id and new_server_id
		-- are both in the set.
		update rhnKickstartSession
			set old_server_id = null
			where old_server_id = s.id;
		update rhnKickstartSession
			set new_server_id = null
			where new_server_id = s.id;
		rhn_channel.clear_subscriptions(s.id,1);
		-- We're deleting everything with a foreign key to rhnServer
		-- here, now.  I'm hoping this will help aleviate our deadlock
		-- problem.

		delete from rhnActionConfigChannel where server_id = s.id;
		delete from rhnActionConfigRevision where server_id = s.id;
		delete from rhnActionPackageRemovalFailure where server_id = s.id;
		delete from rhnChannelFamilyLicenseConsent where server_id = s.id;
		delete from rhnClientCapability where server_id = s.id;
		delete from rhnCpu where server_id = s.id;
		-- there's still a cascade here, because the constraint keeps the
		-- table locked for too long to rebuild it.  Ugh...
		delete from rhnDevice where server_id = s.id;
		delete from rhnProxyInfo where server_id = s.id;
		delete from rhnRam where server_id = s.id;
		delete from rhnRegToken where server_id = s.id;
		delete from rhnSNPServerQueue where server_id = s.id;
		delete from rhnSatelliteChannelFamily where server_id = s.id;
		delete from rhnSatelliteInfo where server_id = s.id;
		-- this cascades to rhnActionConfigChannel and rhnActionConfigFileName
		delete from rhnServerAction where server_id = s.id;
		delete from rhnServerActionPackageResult where server_id = s.id;
		delete from rhnServerActionScriptResult where server_id = s.id;
		delete from rhnServerActionVerifyResult where server_id = s.id;
		delete from rhnServerActionVerifyMissing where server_id = s.id;
		-- counts are handled above.  this should be a delete_ function.
		delete from rhnServerChannel where server_id = s.id;
		delete from rhnServerConfigChannel where server_id = s.id;
		delete from rhnServerCustomDataValue where server_id = s.id;
		delete from rhnServerDMI where server_id = s.id;
		delete from rhnServerMessage where server_id = s.id;
		-- this gets rhnServerMessage (only) on cascade; it's handled just above
		delete from rhnServerEvent where server_id = s.id;
		delete from rhnServerHistory where server_id = s.id;
		delete from rhnServerInfo where server_id = s.id;
		delete from rhnServerInstallInfo where server_id = s.id;
		delete from rhnServerLocation where server_id = s.id;
		delete from rhnServerLock where server_id = s.id;
		delete from rhnServerNeededPackageCache where server_id = s.id;
		delete from rhnServerNeededErrataCache where server_id = s.id;
		delete from rhnServerNetwork where server_id = s.id;
		delete from rhnServerNotes where server_id = s.id;
		-- I'm not removing the foreign key from rhnServerPackage; that'll
		-- take forever.  Do the delete anyway.
		delete from rhnServerPackage where server_id = s.id;
		delete from rhnServerTokenRegs where server_id = s.id;
		delete from rhnSnapshotTag where server_id = s.id;
		-- this cascades to:
		--   rhnSnapshotChannel, rhnSnapshotConfigChannel, rhnSnapshotPackage,
		--   rhnSnapshotConfigRevision, rhnSnapshotServerGroup,
		--   rhnSnapshotTag.
		-- We may want to consider delete_snapshot() at some point, but
		--   I don't think we need to yet.
		delete from rhnSnapshot where server_id = s.id;
		delete from rhnTransaction where server_id = s.id;
		delete from rhnUserServerPrefs where server_id = s.id;
		-- hrm, this one's interesting... we _probably_ should delete
		-- everything for the parent server_id when we delete the proxy,
		-- but we don't currently.
		delete from rhnServerPath where s.id in (server_id, proxy_server_id);
		delete from rhnUserServerPerms where server_id = s.id;

		delete from rhn_interface_monitoring where server_id = s.id;
		delete from rhnServerNetInterface where server_id = s.id;
		delete from rhn_server_monitoring_info where recid = s.id;

		delete from rhnAppInstallSession where server_id = s.id;
		delete from rhnServerUuid where server_id = s.id;

        DELETE FROM rhn_probe_state PS WHERE PS.probe_id in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.host_id = s.id
        );

        DELETE FROM rhn_probe P  WHERE P.recid in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.host_id = s.id
        );

        -- Now we delete any probes that were using this Server
        -- as a Proxy Scout.
        DELETE
          FROM rhn_probe_state PS
         WHERE PS.probe_id in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.sat_cluster_id =
        (SELECT SN.sat_cluster_id
           FROM rhn_sat_node SN
          WHERE SN.server_id = s.id));

        DELETE FROM rhn_probe P
         WHERE P.recid in
        (SELECT CP.probe_id
          FROM rhn_check_probe CP
           WHERE CP.sat_cluster_id =
        (SELECT SN.sat_cluster_id
           FROM rhn_sat_node SN
           WHERE SN.server_id = s.id));

        delete from rhn_check_probe where host_id = s.id;
        delete from rhn_host_probe where host_id = s.id;

        delete from rhn_sat_cluster where recid in
         ( select sat_cluster_id from rhn_sat_node where server_id = s.id );
	delete from rhn_sat_node where server_id = s.id;

		delete from rhnServer
			where id = s.id;
		delete from rhnSet
			where user_id in (
				select	wc.id
				from	rhnServer rs,
					web_contact wc
				where	rs.id = s.id
					and rs.org_id = wc.org_id
			)
			and label = 'system_list'
			and element = s.id;
	end loop;
end delete_server_bulk;
/
show errors;

-- Views
CREATE OR REPLACE FORCE VIEW RHNENTITLEDSERVERS
(ID, ORG_ID, DIGITAL_SERVER_ID, SERVER_ARCH_ID, OS,
 RELEASE, NAME, DESCRIPTION, INFO, SECRET)
AS
select distinct
    S.id,
    S.org_id,
    S.digital_server_id,
    S.server_arch_id,
    S.os,
    S.release,
    S.name,
    S.description,
    S.info,
    S.secret
from
    rhnServerGroup SG,
    rhnServerGroupType SGT,
    rhnServerGroupMembers SGM,
    rhnServer S
where
    S.id = SGM.server_id
and SG.id = SGM.server_group_id
and SGT.label IN ('sw_mgr_entitled', 'enterprise_entitled', 'provisioning_entitled')
and SG.group_type = SGT.id
and SG.org_id = S.org_id;


CREATE OR REPLACE FORCE VIEW RHNSERVERFEATURESVIEW
(SERVER_ID, LABEL)
AS
select
   distinct
   sgm.server_id,
   f.label
from
   rhnServerGroupType sgt,
   rhnServerGroup sg,
   rhnServerGroupMembers sgm,
   rhnServerGroupTypeFeature sgtf,
   rhnFeature f
where
   sg.id = sgm.server_group_id
   and sg.group_type = sgt.id
   and sgt.label in (
               'sw_mgr_entitled','enterprise_entitled',
               'provisioning_entitled',
               'monitoring_entitled'
               )
   and sgt.id = sgtf.server_group_type_id
   and sgtf.feature_id = f.id;

CREATE OR REPLACE FORCE VIEW RHNWEBCONTACTDISABLED
(ID, ORG_ID, LOGIN, LOGIN_UC, PASSWORD,
 OLD_PASSWORD, ORACLE_CONTACT_ID, CREATED, MODIFIED, IGNORE_FLAG)
AS
select
   wcon.id,
   wcon.org_id,
   wcon.login,
   wcon.login_uc,
   wcon.password,
   wcon.old_password,
   wcon.oracle_contact_id,
   wcon.created,
   wcon.modified,
   wcon.ignore_flag
from
   rhnWebContactChangeLog   wccl,
   rhnWebContactChangeState wccs,
   web_contact              wcon
where 1=1
   and wccl.change_state_id = wccs.id
   and wccs.label = 'disabled'
   and wccl.web_contact_id = wcon.id
   and wccl.date_completed =
              (select max(wccl_exists.date_completed)
                 from rhnWebContactChangeLog   wccl_exists
                where wccl.web_contact_id = wccl_exists.web_contact_id);

CREATE OR REPLACE FORCE VIEW RHNWEBCONTACTENABLED
(ID, ORG_ID, LOGIN, LOGIN_UC, PASSWORD,
 OLD_PASSWORD, ORACLE_CONTACT_ID, CREATED, MODIFIED, IGNORE_FLAG)
AS
select
   wcon.id,
   wcon.org_id,
   wcon.login,
   wcon.login_uc,
   wcon.password,
   wcon.old_password,
   wcon.oracle_contact_id,
   wcon.created,
   wcon.modified,
   wcon.ignore_flag
from
   web_contact wcon
minus
select
   wcd.id,
   wcd.org_id,
   wcd.login,
   wcd.login_uc,
   wcd.password,
   wcd.old_password,
   wcd.oracle_contact_id,
   wcd.created,
   wcd.modified,
   wcd.ignore_flag
from
   rhnWebContactDisabled wcd;

CREATE OR REPLACE FORCE VIEW RHNPRIVATEERRATAMAIL
(LOGIN, LOGIN_UC, EMAIL, USER_ID, SERVER_ID,
 ORG_ID, SERVER_NAME, SERVER_ARCH, SERVER_RELEASE, ERRATA_ID,
 ADVISORY)
AS
select
   w.login,
   w.login_uc,
   wpi.email,
   w.id user_id,
   s.id server_id,
   w.org_id org_id,
   s.name server_name,
   sa.name server_arch,
   s.release server_release,
   ce.errata_id errata_id,
   e.advisory
from
   rhnServer s,
   web_user_personal_info wpi,
   rhnUserInfo ui,
   rhnErrata e,
   rhnServerArch sa,
   rhnChannelErrata ce,
   web_contact w,
   rhnServerChannel sc,
   rhnServerGroupMembers sgm,
   rhnServerGroup sg
where 1=1
   and sg.id = sgm.server_group_id
   and sgm.server_id = sc.server_id
   and sg.org_id = w.org_id
   and not exists (
      select   usprefs.server_id
               from  rhnUserServerPrefs usprefs
         where 1=1
         and w.id = usprefs.user_id
               and sc.server_id = usprefs.server_id
               and usprefs.name = 'receive_notifications'
   )
   and w.id = wpi.web_user_id
   and wpi.email is not null
   and w.id = ui.user_id
      and ui.email_notify = 1
   and exists (
      select   1
      from
         rhnUserGroupType  ugt,
         rhnUserGroup      ug,
         rhnUserGroupMembers  ugm
      where 1=1
         and ugt.label = 'org_admin'
         and ugt.id = ug.group_type
         and ug.id = ugm.user_group_id
         and ugm.user_id = w.id
      union all
      select   1
      from
         rhnServerGroupMembers   sq_sgm,
         rhnUserServerGroupPerms usg
      where sc.server_id = sq_sgm.server_id
         and sq_sgm.server_group_id = usg.server_group_id
         and usg.user_id = w.id
   )
   and sc.channel_id = ce.channel_id
   and sc.server_id = s.id
      and exists (
         select 1
      from
            rhnPackageEVR        p_evr,
            rhnPackageEVR        sp_evr,
            rhnServerPackage     sp,
            rhnChannelPackage    cp,
            rhnPackage        p,
            rhnErrataPackage     ep,
            rhnServerPackageArchCompat spac
      where 1=1
         and sc.channel_id = cp.channel_id
         and cp.package_id = p.id
         and ce.errata_id = ep.errata_id
         and ep.package_id = p.id
         and sc.channel_id = ce.channel_id
         and ce.errata_id = ep.errata_id
         and sc.server_id = sp.server_id
         and sp.name_id = p.name_id
         and sp.evr_id = sp_evr.id
         and p.evr_id = p_evr.id
         and sp.evr_id != p.evr_id
         and sp_evr.evr < p_evr.evr
         and sp_evr.evr = (
            select   max(sq2_pe.evr)
            from  rhnServerPackage  sq2_sp,
               rhnPackageEVR     sq2_pe
            where sp.server_id = sq2_sp.server_id
               and sp.name_id = sq2_sp.name_id
               and sq2_sp.evr_id = sq2_pe.id
         )
         and p.package_arch_id = spac.package_arch_id
         and s.server_arch_id = spac.server_arch_id
   )
   and s.server_arch_id = sa.id
   and ce.errata_id = e.id
   and not exists ( select 1
                      from rhnWebContactDisabled wcd
                     where wcd.id = w.id );

CREATE OR REPLACE FORCE VIEW RHNSERVERGROUPOVERVIEW
(ORG_ID, SECURITY_ERRATA, BUG_ERRATA, ENHANCEMENT_ERRATA, GROUP_ID,
 GROUP_NAME, GROUP_ADMINS, SERVER_COUNT, NOTE_COUNT, MODIFIED,
 MAX_MEMBERS)
AS
SELECT SG.org_id,
         (SELECT COUNT(distinct E.id)
            FROM rhnErrata E,
                 rhnServerNeededPackageCache SNPC,
                 rhnServerGroupMembers SGM
           WHERE E.advisory_type = 'Security Advisory'
                 and snpc.errata_id = e.id
                 and snpc.server_id = sgm.server_id
                 and sgm.server_group_id = sg.id
                 AND EXISTS ( SELECT 1
                              FROM rhnServerFeaturesView SFV
                              WHERE SFV.server_id = SGM.server_id
                                    AND SFV.label = 'ftr_system_grouping')),
         (SELECT COUNT(distinct E.id)
            FROM rhnErrata E,
                 rhnServerNeededPackageCache SNPC,
                 rhnServerGroupMembers SGM
           WHERE E.advisory_type = 'Bug Fix Advisory'
                 and snpc.errata_id = e.id
                 and snpc.server_id = sgm.server_id
                 and sgm.server_group_id = sg.id
                 AND EXISTS ( SELECT 1
                              FROM rhnServerFeaturesView SFV
                              WHERE SFV.server_id = SGM.server_id
                                    AND SFV.label = 'ftr_system_grouping')),
         (SELECT COUNT(distinct E.id)
            FROM rhnErrata E,
                 rhnServerNeededPackageCache SNPC,
                 rhnServerGroupMembers SGM
           WHERE E.advisory_type = 'Product Enhancement Advisory'
                 and snpc.errata_id = e.id
                 and snpc.server_id = sgm.server_id
                 and sgm.server_group_id = sg.id
                 AND EXISTS ( SELECT 1
                              FROM rhnServerFeaturesView SFV
                              WHERE SFV.server_id = SGM.server_id
                                    AND SFV.label = 'ftr_system_grouping')),
         SG.id, SG.name,
         (SELECT COUNT(*) FROM rhnUserManagedServerGroups UMSG WHERE UMSG.server_group_id = SG.id),
         (SELECT COUNT(*) FROM rhnServerGroupMembers SGM WHERE SGM.server_group_id = SG.id
                 AND EXISTS ( SELECT 1
                              FROM rhnServerFeaturesView SFV
                              WHERE SFV.server_id = SGM.server_id
                                    AND SFV.label = 'ftr_system_grouping')),
         0, SYSDATE, MAX_MEMBERS
    FROM rhnServerGroup SG;

CREATE OR REPLACE FORCE VIEW RHNSERVEROVERVIEW
(ORG_ID, SERVER_ID, SERVER_NAME, NOTE_COUNT, MODIFIED,
 SERVER_ADMINS, GROUP_COUNT, CHANNEL_LABELS, HISTORY_COUNT, SECURITY_ERRATA,
 BUG_ERRATA, ENHANCEMENT_ERRATA, OUTDATED_PACKAGES, LAST_CHECKIN_DAYS_AGO, LAST_CHECKIN,
 PENDING_UPDATES, OS, RELEASE, SERVER_ARCH_NAME, LOCKED)
AS
select
    s.org_id, s.id, s.name, 0, s.modified,
    ( select count(user_id) from rhnUserServerPerms ap
      where server_id = s.id ),
    ( select count(server_group_id) from rhnVisibleServerGroupMembers
      where server_id = s.id ),
    NVL(( select C.name
        from rhnChannel C,
        rhnServerChannel SC
       where SC.server_id = S.id
         and SC.channel_id = C.id
    and C.parent_channel IS NULL), '(none)'),
    ( select count(id) from rhnServerHistory
      where
            server_id = S.id),
    ( select count(*) from rhnServerErrataTypeView setv
      where
            setv.server_id = s.id
        and setv.errata_type = 'Security Advisory'),
    ( select count(*) from rhnServerErrataTypeView setv
      where
            setv.server_id = s.id
        and setv.errata_type = 'Bug Fix Advisory'),
    ( select count(*) from rhnServerErrataTypeView setv
      where
            setv.server_id = s.id
        and setv.errata_type = 'Product Enhancement Advisory'),
    ( select count(distinct p.name_id) from rhnPackage p, rhnServerNeededPackageCache snpc
      where
             snpc.server_id = S.id
    and p.id = snpc.package_id
    ),
    ( select sysdate - checkin from rhnServerInfo where server_id = S.id ),
    ( select TO_CHAR(checkin, 'YYYY-MM-DD HH24:MI:SS') from rhnServerInfo where server_id = S.id ),
    ( select count(1)
        from rhnServerAction
       where server_id = S.id
         and status in (0, 1)),
    os,
    release,
    ( select name from rhnServerArch where id = s.server_arch_id),
    NVL((select 1 from rhnServerLock SL WHERE SL.server_id = S.id), 0)
from
    rhnServer S;


-- Convert Prov Servers
declare
   i number := 0;

   mgmt_group_id number;

   mgmt_group_type number;

   cursor prov_orgs is
   select distinct org_id
   from rhnServerGroup
   where group_type = lookup_sg_type ( 'provisioning_entitled' )
   and max_members > 0 and current_members > 0;

   cursor prov_servers (org_id_in in number) is
   select sgm.server_id, sg.org_id
     from rhnServerGroupMembers sgm,
          rhnServerGroup sg
    where sgm.server_group_id = sg.id
      and sg.group_type = lookup_sg_type ('provisioning_entitled')
      and sg.org_id = org_id_in;

begin
   mgmt_group_type := lookup_sg_type ('enterprise_entitled');

   for prov_org in prov_orgs loop
      select sg.id
        into mgmt_group_id
        from rhnServerGroup sg
       where sg.org_id = prov_org.org_id
         and sg.group_type = mgmt_group_type;

      for prov_server in prov_servers (prov_org.org_id) loop

         rhn_server.insert_into_servergroup (prov_server.server_id, mgmt_group_id);

      end loop;

   end loop;

end;
/
show errors;

-- Convert any tar channels to i386
declare
   cursor tar_channels is
   select id
   from rhnChannel
   where channel_arch_id in ( lookup_channel_arch ( 'tar-gnu' ),
                              lookup_channel_arch ( 'channel-patches-sparc-sun-solaris' ) );

begin

   for tar_channel in tar_channels loop
      update rhnChannel
      set channel_arch_id = lookup_channel_arch ( 'channel-ia32' )
      where id = tar_channel.id;

      dbms_output.put_line ('updated channel id ' || tar_channel.id);
   end loop;

end;
/
show errors;

declare
   schema_name varchar2(30);
begin
   select owner
   into schema_name
   from all_tables
   where table_name = 'RHNSERVER';

   dbms_utility.compile_schema (schema_name);
end;
/
show errors;

alter type evr_t compile body;

-- Data changes
-- Provisioning is not a base entitlement
update rhnServerGroupType
set is_base = 'N'
where label = 'provisioning_entitled';

insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_package_updates', 'Update Packages',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_errata_updates', 'Errata Updates',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_hardware_refresh', 'Refresh Hardware Profile',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_package_refresh', 'Refresh Packages Profile',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_package_remove', 'Remove Packages',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_auto_errata_updates', 'Auto Errata Update',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_system_grouping', 'System Groups',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_package_verify', 'Verify Packages',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_profile_compare', 'Compare Package Profiles',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_proxy_capable', 'Proxy Capable',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_sat_capable', 'Satellite Capable',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_reboot', 'Reboot System',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_satellite_applet', 'Satellite Applet',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_osa_bus', 'OSA Bus',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_kickstart', 'Kickstart',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_config', 'Config File Management',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_custom_info', 'Custom Information',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_delta_action', 'Package Profile Sync',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_snapshotting', 'System Snapshotting',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_agent_smith', 'Agent Smith',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_remote_command', 'Execute Remote Command',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_schedule_probe', 'Schedule Probe',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_probes', 'Probes',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_nonlinux_support', 'Non-Linux Support',
        sysdate, sysdate);
insert into rhnFeature (id, label, name, created, modified)
values (rhn_feature_seq.nextval, 'ftr_daily_summary', 'Daily Summary',
        sysdate, sysdate);

delete from rhnArchTypeActions
where arch_type_id = ( select archt.id from rhnArchType archt where ArchT.label = 'solaris-patch-cluster' )
  and action_style = 'remove';

delete from rhnArchTypeActions
where arch_type_id = ( select archt.id from rhnArchType archt where ArchT.label = 'solaris-patch' )
  and action_style = 'remove';

INSERT
   INTO rhnArchTypeActions (arch_type_id, action_style, action_type_id)
        (SELECT ArchT.id, 'remove', ActionT.id
           FROM rhnArchType ArchT, rhnActionType ActionT
    WHERE ArchT.label = 'solaris-patch-cluster'
      AND ActionT.label = 'solarispkgs.patchClusterRemove');

INSERT
   INTO rhnArchTypeActions (arch_type_id, action_style, action_type_id)
        (SELECT ArchT.id, 'remove', ActionT.id
           FROM rhnArchType ArchT, rhnActionType ActionT
    WHERE ArchT.label = 'solaris-patch'
      AND ActionT.label = 'solarispkgs.patchRemove');


insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'i386-solaris', 'i386-solaris', lookup_arch_type('sysv-solaris')
   from dual
   where not exists ( select 1
                        from rhnPackageArch
                       where label = 'i386-solaris'
                         and name = 'i386-solaris'
                         and arch_type_id = lookup_arch_type('sysv-solaris')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'nosrc', 'nosrc', lookup_arch_type('rpm')
   from dual
   where not exists ( select 1
                        from rhnPackageArch
                       where label = 'nosrc'
                         and name = 'nosrc'
                         and arch_type_id = lookup_arch_type('rpm')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'sparc-solaris-patch', 'Sparc Solaris patch', lookup_arch_type('solaris-patch')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'sparc-solaris-patch'
                         and name = 'Sparc Solaris patch'
                         and arch_type_id = lookup_arch_type('solaris-patch')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'i386-solaris-patch', 'i386 Solaris patch', lookup_arch_type('solaris-patch')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'i386-solaris-patch'
                         and name = 'i386 Solaris patch'
                         and arch_type_id = lookup_arch_type('solaris-patch')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'sparc-solaris-patch-cluster', 'Sparc Solaris patch cluster', lookup_arch_type('solaris-patch-cluster')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'sparc-solaris-patch-cluster'
                         and name = 'Sparc Solaris patch cluster'
                         and arch_type_id = lookup_arch_type('solaris-patch-cluster')
                    )
);


insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'i386-solaris-patch-cluster', 'i386 Solaris patch cluster', lookup_arch_type('solaris-patch-cluster')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'i386-solaris-patch-cluster'
                         and name = 'i386 Solaris patch cluster'
                         and arch_type_id = lookup_arch_type('solaris-patch-cluster')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'noarch-solaris', 'noarch-solaris', lookup_arch_type('sysv-solaris')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'noarch-solaris'
                         and name = 'noarch-solaris'
                         and arch_type_id = lookup_arch_type('sysv-solaris')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'noarch-solaris-patch', 'noarch-solaris-patch', lookup_arch_type('sysv-solaris')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'noarch-solaris-patch'
                         and name = 'noarch-solaris-patch'
                         and arch_type_id = lookup_arch_type('sysv-solaris')
                    )
);

insert into rhnPackageArch (id, label, name, arch_type_id)
(
   select rhn_package_arch_id_seq.nextval, 'noarch-solaris-patch-cluster', 'noarch-solaris-patch-cluster', lookup_arch_type('sysv-solaris')
     from dual
    where not exists ( select 1
                        from rhnPackageArch
                       where label = 'noarch-solaris-patch-cluster'
                         and name = 'noarch-solaris-patch-cluster'
                         and arch_type_id = lookup_arch_type('sysv-solaris')
                    )
);


insert into rhnServerArch (id, label, name, arch_type_id)
(
   select rhn_server_arch_id_seq.nextval, 'i386-i86pc-solaris', 'i386 Solaris', lookup_arch_type('sysv-solaris')
     from dual
    where not exists ( select 1
                         from rhnServerArch
                        where label = 'i386-i86pc-solaris'
                          and name = 'i386 Solaris'
                          and arch_type_id = lookup_arch_type('sysv-solaris')
                     )
);

delete from rhnChannelPackageArchCompat
where channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-patches-sparc-sun-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch');

delete from rhnChannelPackageArchCompat
where channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-patches-sparc-sun-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch-cluster');

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-redhat-linux'), LOOKUP_PACKAGE_ARCH('i386'), 0, sysdate, sysdate
  from dual
  where not exists ( select 1
                       from rhnServerPackageArchCompat spac
                      where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-redhat-linux')
                        and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386')
                   )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-redhat-linux'), LOOKUP_PACKAGE_ARCH('noarch'), 1000, sysdate, sysdate
  from dual
  where not exists ( select 1
                       from rhnServerPackageArchCompat spac
                      where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-redhat-linux')
                        and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch')
                   )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i486-redhat-linux'), LOOKUP_PACKAGE_ARCH('i486'), 0, sysdate, sysdate
  from dual
  where not exists ( select 1
                       from rhnServerPackageArchCompat spac
                      where spac.server_arch_id = LOOKUP_SERVER_ARCH('i486-redhat-linux')
                        and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i486')
                   )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i486-redhat-linux'), LOOKUP_PACKAGE_ARCH('i386'), 10, sysdate, sysdate
  from dual
  where not exists ( select 1
                       from rhnServerPackageArchCompat spac
                      where spac.server_arch_id = LOOKUP_SERVER_ARCH('i486-redhat-linux')
                        and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386')
                   )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4m-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris'), 410, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4m-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch'), 510, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4m-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster'), 610, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
                      )
);

-- insert into rhnServerPackageArchCompat
-- (server_arch_id, package_arch_id, preference) values
-- (LOOKUP_SERVER_ARCH('sparc-sun4u-solaris'), LOOKUP_PACKAGE_ARCH('sparc.sun4u-solaris'), 10);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris'), 10, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris-patch'), 200, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris-patch')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris-patch-cluster'), 300, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris-patch-cluster')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris'), 410, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch'), 510, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster'), 610, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
                      )
);

delete from rhnServerPackageArchCompat
where server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch');

delete from rhnServerPackageArchCompat
where server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch-cluster');

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4m-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch'), 210, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4m-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster'), 310, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4m-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster')
                      )
);

delete from rhnServerPackageArchCompat
where server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4u-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch');

delete from rhnServerPackageArchCompat
where server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4u-solaris')
  and package_arch_id = LOOKUP_PACKAGE_ARCH('solaris-patch-cluster');

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4u-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch'), 210, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4u-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch')
                      )
);

insert into rhnServerPackageArchCompat
( select LOOKUP_SERVER_ARCH('sparc-sun4u-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster'), 310, sysdate, sysdate
    from dual
    where not exists ( select 1 from rhnServerPackageArchCompat spac
                        where spac.server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4u-solaris')
                          and spac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster')
                      )
);

delete from rhnPackageArch where label = 'solaris-patch';
delete from rhnPackageArch where label = 'solaris-patch-cluster';


-- delete from rhnServerChannelArchCompat
-- where server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
--   and channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris');

insert into rhnChannelArch (id, label, name, arch_type_id)
( select rhn_channel_arch_id_seq.nextval, 'channel-i386-sun-solaris', 'i386 Solaris', lookup_arch_type('sysv-solaris')
  from dual
  where not exists ( select 1
                       from rhnChannelArch
                      where label = 'channel-i386-sun-solaris'
                        and name = 'i386 Solaris'
                   )
);


delete from rhnServerChannelArchCompat
where server_arch_id = LOOKUP_SERVER_ARCH('sparc-sun4u-solaris')
  and channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-patches-sparc-sun-solaris');

insert into rhnServerChannelArchCompat (server_arch_id, channel_arch_id)
( select LOOKUP_SERVER_ARCH('i386-i86pc-solaris'), LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
    from dual
    where not exists ( select 1 from rhnServerChannelArchCompat scac
                        where scac.server_arch_id = LOOKUP_SERVER_ARCH('i386-i86pc-solaris')
                          and scac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-ia32'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-ia32')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-ia64'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-ia64')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-alpha'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-alpha')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-s390'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-s390')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-s390x'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-s390x')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-iSeries'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-iSeries')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-pSeries'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-pSeries')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-x86_64'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-x86_64')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-ppc'), LOOKUP_PACKAGE_ARCH('nosrc')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-ppc')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('nosrc')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris-patch-cluster')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris-patch-cluster')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('i386-solaris-patch')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('i386-solaris-patch')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-i386-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris'), LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('noarch-solaris-patch-cluster')
                      )
);

-- delete from rhnChannelArch where label = 'channel-i386-sun-solaris';
delete from rhnChannelArch where label = 'tar-gnu';

update rhn_command_parameter
set default_value = null
where command_id = 16 and param_name = 'genericGTwarning';

update rhn_command_parameter
set default_value = null
where command_id = 16 and param_name = 'genericGTcritical';

update rhn_command_parameter
set default_value = null
where command_id = 25 and param_name = 'warn';

update rhn_command_parameter
set default_value = null
where command_id = 25 and param_name = 'critical';

update rhn_command_parameter
set default_value = 0, data_type_name = 'checkbox', description = 'Include reclaimable memory',
    field_widget_name = 'checkbox'
where command_id = 25 and param_name = 'reclaim';

delete from rhn_command_parameter
where command_id = 56 and param_name = 'virtual_host';

update rhn_command_parameter
set default_value = null
where command_id = 92 and param_name = 'warn';

update rhn_command_parameter
set default_value = null
where command_id = 92 and param_name = 'critical';

update rhn_command_parameter
set default_value = null
where command_id = 95 and param_name = 'warnabove';

update rhn_command_parameter
set default_value = null
where command_id = 95 and param_name = 'criticalabove';

update rhn_command_parameter
set default_value = null
where command_id = 95 and param_name = 'warnbelow';

update rhn_command_parameter
set default_value = null
where command_id = 95 and param_name = 'criticalbelow';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nblocked_warn';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nblocked_critical';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nchildren_warn';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'ndefunct_warn';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'ndefunct_critical';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nstopped_warn';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nstopped_critical';

update rhn_command_parameter
set default_value = null, description = 'Warning Sleeping Process Count'
where command_id = 230 and param_name = 'nswapped_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Sleeping Process Count'
where command_id = 230 and param_name = 'nswapped_critical';

update rhn_command_parameter
set default_value = null
where command_id = 230 and param_name = 'nchildren_critical';

delete from rhn_command_parameter
where command_id = 231 and param_name = 'cust_0';

delete from rhn_command_parameter
where command_id = 231 and param_name = 'asset_0';

update rhn_command_parameter
set default_value = null, description = 'Warning Blocked Process Count'
where command_id = 231 and param_name = 'nblocked_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Blocked Process Count'
where command_id = 231 and param_name = 'nblocked_critical';

update rhn_command_parameter
set default_value = null, description = 'Warning Child Process Count'
where command_id = 231 and param_name = 'nchildren_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Child Process Count'
where command_id = 231 and param_name = 'nchildren_critical';

update rhn_command_parameter
set default_value = null, description = 'Warning Defunct Process Count'
where command_id = 231 and param_name = 'ndefunct_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Defunct Process Count '
where command_id = 231 and param_name = 'ndefunct_critical';

update rhn_command_parameter
set default_value = null, description = 'Warning Stopped Process Count'
where command_id = 231 and param_name = 'nstopped_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Stopped Process Count'
where command_id = 231 and param_name = 'nstopped_critical';

update rhn_command_parameter
set default_value = null, description = 'Warning Sleeping Process Count'
where command_id = 231 and param_name = 'nswapped_warn';

update rhn_command_parameter
set default_value = null, description = 'Critical Sleeping Process Count'
where command_id = 231 and param_name = 'nswapped_critical';

update rhn_command_parameter p
set p.default_value = null
where p.command_id = 22
  and p.param_name in ('critical','warn');

insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class)
    values ( 10,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::FTP');

insert into rhnException values (-20283, 'invalid_feature', 'The specified feature does not exist');

insert into rhnException
values (-20284, 'invalid_base_entitlement', 'The base entitlement is not valid for adding on other entitlements');

insert into rhnException
values (-20285, 'invalid_addon_entitlement', 'The addon entitlement is not valid for adding onto other entitlements');

insert into rhnException
values (-20286, 'no_available_server_group',
         'The server can be entitled to that entitlement, but no valid server group was found');

insert into rhnException
values (-20287, 'invalid_entitlement', 'The server can not be entitled to the specified level');

insert into rhnException
values (-20288, 'invalid_server_group_member',
         'The specified entitlement can not be removed from the specified server because the server does not have that entitlement level');

insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_package_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_errata_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_hardware_refresh'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_package_refresh'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_package_remove'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_auto_errata_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('sw_mgr_entitled'), lookup_feature_type('ftr_daily_summary'),
        sysdate,sysdate);


insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_package_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_errata_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_hardware_refresh'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_package_refresh'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_package_remove'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_auto_errata_updates'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_system_grouping'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_package_verify'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_profile_compare'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_proxy_capable'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_sat_capable'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_reboot'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_satellite_applet'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_osa_bus'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('enterprise_entitled'), lookup_feature_type('ftr_daily_summary'),
        sysdate,sysdate);


insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_kickstart'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_config'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_custom_info'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_delta_action'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_snapshotting'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_agent_smith'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('provisioning_entitled'), lookup_feature_type('ftr_remote_command'),
        sysdate,sysdate);

insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('monitoring_entitled'), lookup_feature_type('ftr_schedule_probe'),
        sysdate,sysdate);
insert into rhnServerGroupTypeFeature (server_group_type_id, feature_id,
                                       created, modified)
values (lookup_sg_type('monitoring_entitled'), lookup_feature_type('ftr_probes'),
        sysdate,sysdate);

insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Generic Patch', 'generic');
insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Kernel Update Patch', 'kernel');
insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Restricted Patch', 'restricted');
insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Point Patch', 'point');
insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Temporary Patch', 'temporary');
insert into rhnSolarisPatchType (id, name, label)
   values (rhn_solaris_pt_seq.nextval, 'Nonstandard Patch', 'nonstandard');

insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('enterprise_entitled'), lookup_sg_type('monitoring_entitled'));

insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('enterprise_entitled'), lookup_sg_type('provisioning_entitled'));

insert into rhnWebContactChangeState (id, label) values (rhn_wcon_change_state_seq.nextval, 'enabled');
insert into rhnWebContactChangeState (id, label) values (rhn_wcon_change_state_seq.nextval, 'disabled');

update rhnServerGroupType set permanent = 'N';

update rhnServerGroupType set is_base = 'N'
where label in ( 'monitoring_entitled' );

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('sparc-sun4m-solaris'), lookup_sg_type('enterprise_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('sparc-sun4m-solaris')
                          and ssgac.server_group_type = lookup_sg_type('enterprise_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('sparc-sun4u-solaris'), lookup_sg_type('enterprise_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('sparc-sun4u-solaris')
                          and ssgac.server_group_type = lookup_sg_type('enterprise_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i386-i86pc-solaris'), lookup_sg_type('enterprise_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i386-i86pc-solaris')
                          and ssgac.server_group_type = lookup_sg_type('enterprise_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('sparc-sun4m-solaris'), lookup_sg_type('provisioning_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('sparc-sun4m-solaris')
                          and ssgac.server_group_type = lookup_sg_type('provisioning_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('sparc-sun4u-solaris'), lookup_sg_type('provisioning_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('sparc-sun4u-solaris')
                          and ssgac.server_group_type = lookup_sg_type('provisioning_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i386-i86pc-solaris'), lookup_sg_type('provisioning_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i386-i86pc-solaris')
                          and ssgac.server_group_type = lookup_sg_type('provisioning_entitled')
                      )
);


delete from rhnServerServerGroupArchCompat
where server_arch_id = lookup_server_arch('sparc-sun4m-solaris')
  and server_group_type = lookup_sg_type('nonlinux_entitled');

delete from rhnServerServerGroupArchCompat
where server_arch_id = lookup_server_arch('sparc-sun4u-solaris')
  and server_group_type = lookup_sg_type('nonlinux_entitled');

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i386-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i386-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i486-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i486-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i586-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i586-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('i686-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('i686-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('athlon-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('athlon-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('alpha-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('alpha-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('alphaev6-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('alphaev6-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ia64-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ia64-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('s390-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('s390-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('s390x-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('s390x-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ppc-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ppc-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ppc64-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ppc64-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('pSeries-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('pSeries-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('iSeries-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('iSeries-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('x86_64-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('x86_64-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ia32e-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ia32e-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('amd64-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('amd64-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ppc64iseries-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ppc64iseries-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type )
( select lookup_server_arch('ppc64pseries-redhat-linux'), lookup_sg_type('monitoring_entitled')
    from dual
    where not exists ( select 1 from rhnServerServerGroupArchCompat ssgac
                        where ssgac.server_arch_id = lookup_server_arch('ppc64pseries-redhat-linux')
                          and ssgac.server_group_type = lookup_sg_type('monitoring_entitled')
                      )
);


delete from rhnServerChannelArchCompat
where server_arch_id = lookup_server_arch('sparc-sun4m-solaris')
  and channel_arch_id = lookup_channel_arch('channel-patches-sparc-sun-solaris');

delete from rhnChannelArch where label = 'channel-patches-sparc-sun-solaris';

insert into DB_CHANGE_RESOURCE_TYPES (RESOURCE_TYPE)
( select 'TABLE PARTITION'
    from dual
   where not exists ( select 1 from db_change_resource_types
                       where resource_type = 'TABLE PARTITION' )
);

insert into DB_CHANGE_RESOURCE_TYPES (RESOURCE_TYPE)
( select 'INDEX PARTITION'
    from dual
   where not exists ( select 1 from db_change_resource_types
                       where resource_type = 'INDEX PARTITION' )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch-cluster')
                      )
);

insert into rhnChannelPackageArchCompat (channel_arch_id, package_arch_id)
( select LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris'), LOOKUP_PACKAGE_ARCH('sparc-solaris-patch')
    from dual
    where not exists ( select 1 from rhnChannelPackageArchCompat cpac
                        where cpac.channel_arch_id = LOOKUP_CHANNEL_ARCH('channel-sparc-sun-solaris')
                          and cpac.package_arch_id = LOOKUP_PACKAGE_ARCH('sparc-solaris-patch')
                      )
);

alter table db_change_script add ( expect_fail char(1) default 0 );
alter table db_change_history add ( elapsed_seconds number (12, 3) default 0 );

create table db_change_script_parsed (
   bug_id   number(12)
            constraint db_csp_bid_nn not null,
   seq_no   number(12)
            constraint db_csp_sno_nn not null,
   stmt_no  number(12)
            constraint db_csp_stno_nn not null,
   line_no  number(12)
            constraint db_csp_lno_nn not null,
   line     varchar2(1000)
            constraint db_csp_line_nn not null
);

COMMENT ON TABLE db_change_script_parsed IS 'DBCSP  Database change script parsed';

ALTER TABLE db_change_script_parsed ADD CONSTRAINT dbcsp_dbcsc_bug_id_seq_no_fk FOREIGN KEY
(
    bug_id,
    seq_no
)
 REFERENCES db_change_script(
    bug_id,
    seq_no
)
ON DELETE CASCADE
NOT DEFERRABLE
INITIALLY IMMEDIATE
ENABLE NOVALIDATE;

create unique index rhn_srvpo_serv_orig_pr_id_uq
   on rhn_service_probe_origins ( origin_probe_id, service_probe_id )
   storage ( freelists 16 )
   initrans 32;

insert into rhnOrgEntitlementType (id, label, name)
( select rhn_org_entitlement_type_seq.nextval, 'rhn_monitor','Red Hat Network Monitoring'
    from dual
   where not exists ( select 1 from rhnOrgEntitlementType oet
                       where oet.label = 'rhn_monitor'
                    )
);
